@@ -617,7 +617,7 @@ GEN detint(GEN x)
 	{cm=1;affii(ggcd(piv,det1),det1);affii(pivprec,piv);rg--;}
       avma=av1;k++;
     }
-  if(rg+cm==m) {tetpil=avma;return gerepile(av,tetpil,gcopy(det1));}
+  if(rg+cm==m) {tetpil=avma;return gerepile(av,tetpil,absi(det1));}
   else {avma=av;return gzero;}
 }
 
@@ -1361,7 +1361,12 @@ GEN hnfspec(long** mat, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,lon
       p1=cgetg(li+1,18);matgen[j]=(long)p1;
       for(i=1;i<=li;i++) p1[i]=lstoi(mat[j][vperm[i]]);
     }
-  if(DEBUGLEVEL>2) output(matgen);
+  if(DEBUGLEVEL>2)
+    {
+      fprintf(errfile,"avant hnfspec :\n");
+      fprintf(errfile,"matgen =\n");
+      output(matgen);
+    }
   matu=cgetg(co+1,19);
   for(j=1;j<=co;j++)
     {
@@ -1397,9 +1402,14 @@ GEN hnfspec(long** mat, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,lon
   for(j=1;j<=co;j++)
     {
       p1=cgetg(li-k0+1,18);matgen[j]=(long)p1;
-      for(i=k0+1;i<=li;i++) p1[i]=lstoi(mat[j][vperm[i]]);
+      for(i=k0+1;i<=li;i++) p1[i-k0]=lstoi(mat[j][vperm[i]]);
+    }
+  if(DEBUGLEVEL>3)
+    {
+      fprintf(errfile,"apres phase1 :\n");
+      fprintf(errfile,"matgen =\n");
+      output(matgen);
     }
-  if(DEBUGLEVEL>2) output(matgen);
   fl=1;sizemax=0;
   while((lig>lk0)&&fl&&(sizemax<=((long)MAXULONG>>2)))
     {
@@ -1456,13 +1466,6 @@ GEN hnfspec(long** mat, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,lon
       vmax[j]=s;
     }
   fl=1;
-  matgen=cgetg(co+1,19);
-  for(j=1;j<=co;j++)
-    {
-      p1=cgetg(li-k0+1,18);matgen[j]=(long)p1;
-      for(i=k0+1;i<=li;i++) p1[i]=lstoi(mat[j][vperm[i]]);
-    }
-  if(DEBUGLEVEL>2) output(matgen);
   while((lig>lk0)&&fl)
     {
       for(i=lig;i>lk0;i--)
@@ -1515,6 +1518,12 @@ GEN hnfspec(long** mat, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,lon
       p1=cgetg(li-k0+1,18);matgen[j]=(long)p1;
       for(i=k0+1;i<=li;i++) p1[i-k0]=lstoi(mat[j][vperm[i]]);
     }
+  if(DEBUGLEVEL>3) 
+    {
+      fprintf(errfile,"apres phase2 :\n");
+      fprintf(errfile,"matgen =\n");
+      output(matgen);
+    }
   for(i=li-1;i>lig;i--)
     {
       p2=(GEN)matgen[i+co-li];
@@ -1551,6 +1560,12 @@ GEN hnfspec(long** mat, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,lon
 	  matid+=dec;matgen+=dec;
 	}
     }
+  if(DEBUGLEVEL>3) 
+    {
+      fprintf(errfile,"apres nettoyage identite :\n");
+      fprintf(errfile,"matgen =\n");
+      output(matgen);
+    }
   nlze=lk0-k0;lnz=lig-nlze;
   av2=avma;p1=cgetg(li+1,17);
   for(i=1;i<=nlze;i++) p1[i]=lstoi(vperm[i+k0]);
@@ -1617,6 +1632,13 @@ GEN hnffinal(GEN matgen, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,lo
   long i,j,k,s,i1,j1,j2,av2,tetpil,av,dec;
   int fl;
 
+  if(DEBUGLEVEL>3) 
+    {
+      fprintf(errfile,"avant hnffinal :\n");
+      fprintf(errfile,"mit =\n");output(matgen);
+      if(nlze) {fprintf(errfile,"pdep =\n");output(pdep);}
+      fprintf(errfile,"matalpha =\n");output(matalpha);
+    }
   av=avma;
   u1u2=lllkerim(matgen);u1=(GEN)u1u2[1];u2=(GEN)u1u2[2];
   if(lg(u2)<=lnz) err(maxranker);
@@ -1696,16 +1718,29 @@ GEN hnffinal(GEN matgen, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,lo
     }
   dec=lpile(av,tetpil,0)>>TWOPOTBYTES_IN_LONG;if(nlze) pdepnew+=dec;
   matalphanew+=dec;wpronew+=dec;matc+=dec;*ptmatc=matc;*ptmatalpha=matalphanew;
-  if(nlze) *ptpdep=pdepnew;*ptcol=col-s;return wpronew;
+  if(nlze) *ptpdep=pdepnew;
+  *ptcol=col-s;
+  if(DEBUGLEVEL>3) 
+    {
+      fprintf(errfile,"apres hnffinal :\n");
+      fprintf(errfile,"mit =\n");output(wpronew);
+      if(nlze) {fprintf(errfile,"pdep =\n");output(pdepnew);}
+      fprintf(errfile,"matalpha =\n");output(matalphanew);
+    }
+  return wpronew;
 }
 
 GEN hnfadd(GEN mit, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,long co,long li,long col,long* ptnlze,GEN extramat,GEN extramatc)
 {
   GEN extramat1,extramat2,p1,p2,extramatnew,matalpha=*ptmatalpha,matc=*ptmatc;
   GEN matcnew,matcnew2,matalphac,extramatcnew,p3,p4,matalphanew,pdep=*ptpdep;
-  long i,j,extrarel,sizeofmit,lig,colnew,colshort,tetpil,av=avma,dec,nlze;
+  long i,j,extrarel,sizeofmit,lig,colnew,colshort,tetpil,av=avma,av2,dec,nlze;
   long lp3,*ww,*permpro;
 
+  if(DEBUGLEVEL>3) 
+    {
+      fprintf(errfile,"\nextramat =\n");output(extramat);
+    }
   extrarel=lg(extramat)-1;nlze=*ptnlze;
   sizeofmit=lg(mit)-1;lig=sizeofmit+nlze;
   extramat1=cgetg(extrarel+1,19);extramat2=cgetg(extrarel+1,19);
@@ -1736,6 +1771,10 @@ GEN hnfadd(GEN mit, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,long co
       for(i=1;i<=nlze;i++) p1[i]=coeff(pdep,i,j-extrarel);
       for(i=nlze+1;i<=lig;i++) p1[i]=coeff(mit,i-nlze,j-extrarel);
     }
+  if(DEBUGLEVEL>3) 
+    {
+      fprintf(errfile,"extramat apres traitement =\n");output(extramat);
+    }
   for(j=extrarel+1;j<=co-col+colshort;j++)
     matcnew[j]=matc[j-extrarel+col-sizeofmit];
   p3=imagecompl(gtrans(extramat));lp3=lg(p3)-1;
@@ -1763,7 +1802,11 @@ GEN hnfadd(GEN mit, GEN* ptpdep, GEN* ptmatc,long* vperm,GEN* ptmatalpha,long co
       p2=cgetg(lig+1,18);matalphanew[j]=(long)p2;
       for(i=1;i<=lig;i++) p2[i]=coeff(matalpha,permpro[i],j);
     }
-  matalpha=matalphanew;free(permpro);
+  matalpha=matalphanew;
+  av2=avma;p1=cgetg(lig+1,17);
+  for(i=1;i<=lig;i++) p1[i]=lstoi(vperm[permpro[i]]);
+  for(i=1;i<=lig;i++) vperm[i]=itos((GEN)p1[i]);
+  avma=av2;free(permpro);
   mit=hnffinal(p4,&pdep,&matcnew,vperm,&matalpha,lig-lp3,co-col+colshort,li,colshort,lig,lp3,&colnew);
   tetpil=avma;matalpha=gcopy(matalpha);mit=gcopy(mit);pdep=gcopy(pdep);
   matcnew2=cgetg(co+extrarel+1,19);
