@@ -171,12 +171,12 @@ GEN     invmulmat(GEN a, GEN b)     /* calcule a^(-1)*b, b etant une matrice.
                  
      
 {
-  long  nbli,nbco,i,j,k,av,av1,av2,av3,av4;
+  long  nbli,nbco,i,j,k,av,av1,av2,av3,av4,tp;
   GEN   aa,x,p,m,u;
   
   
   nbco=lg(b)-1;if(!nbco) return cgetg(1,19);
-  nbli=lg((GEN)a[1])-1;
+  nbli=(lg(a)==1)?0:lg((GEN)a[1])-1;
   if (lg(a)-1 != nbli) err(invmuler1);
   if (nbli!=lg((GEN)b[1])-1) err(invmuler1);
   av=avma;
@@ -196,8 +196,8 @@ GEN     invmulmat(GEN a, GEN b)     /* calcule a^(-1)*b, b etant une matrice.
     }
   for (i=1;i<nbli;i++)
     {
-      p=gcoeff(aa,i,i);k=i;
-      if (gcmp0(p))
+      p=gcoeff(aa,i,i);k=i;tp=typ(p);
+      if ((gcmp0(p))||(((tp==2)||(tp==6))&&(gexpo(p)<-((lg(p)-2)<<TWOPOTBITS_IN_LONG))))
 	{
 	  for (k=i+1;(k<=nbli)&&gcmp0(gcoeff(aa,k,i));k++);
 	  if (k>nbli) err(matinv2);
@@ -266,9 +266,10 @@ GEN invmat(GEN a)
 }
 
 
-GEN     invmulmatreel(GEN a, GEN b)     /* calcule a^(-1)*b, b etant une matrice.
-				  ( Il faut : nblig(a)=nbcol(a)=nblig(b) ) */
-                 
+GEN     invmulmatreel(GEN a, GEN b)    
+
+ /* calcule a^(-1)*b, b etant une matrice.
+( Il faut : nblig(a)=nbcol(a)=nblig(b) ) */
      
 {
   long  nbli,nbco,i,j,k,av,av1,av2,av3,av4;
@@ -276,8 +277,9 @@ GEN     invmulmatreel(GEN a, GEN b)     /* calcule a^(-1)*b, b etant une matrice
   
   
   nbco=lg(b)-1;nbli=lg((GEN)a[1])-1;
+  nbli=(lg(a)==1)?0:lg((GEN)a[1])-1;
   if (lg(a)-1 != nbli) err(invmuler1);
-  /* la verif nblig(b)=nblig(a) n'est pas faite ... */
+  if (nbli!=lg((GEN)b[1])-1) err(invmuler1);
   av=avma;
   x=cgetg(nbco+1,19);
   for (j=1;j<=nbco;j++)
@@ -870,6 +872,22 @@ GEN     trace(GEN x)
   return y;
 }
 
+GEN     trace9(GEN x, GEN p1)
+/* a usage interne. Pas de verifs.
+Calcule trace(gmodulcp(x,pol)), ou p1=polsym(pol,lgef(pol)-4) */
+{
+  GEN p2,p3;
+  long av=avma,tetpil,i;
+
+  if(!signe(x)) return gzero;
+  for(p2=gzero,i=2;i<lgef(x);i++)
+    {
+      p3=gmul((GEN)x[i],(GEN)p1[i-1]);tetpil=avma;
+      p2=gadd(p2,p3);
+    }
+  return gerepile(av,tetpil,p2);
+}
+
 /*===================================*/
 /*     Reduction en carres           */
 /*===================================*/
