@@ -72,6 +72,7 @@ entree fonctions[]={
   {"akell",2,(void *)akell,5,0},
   {"algdep",23,(void *)algdep,8,0},
   {"algdep2",33,(void *)algdep2,8,0},
+  {"allocatemem",11,(void *)allocatemem,2,0},
   {"anell",23,(void *)anell,5,0},
   {"apell",2,(void *)apell,5,0},
   {"apell2",2,(void *)apell2,5,0},
@@ -96,12 +97,12 @@ entree fonctions[]={
   {"boundcf",21,(void *)gboundcf,4,0},
   {"boundfact",21,(void *)boundfact,4,0},
   {"box",35,(void *)rectbox,10,0},
-  {"buchgen",4,(void *)buchgen,6,0},
-  {"buchgenfu",4,(void *)buchgenfu,6,0},
-  {"buchimag",4,(void *)buchimag,4,0},
-  {"buchinit",4,(void *)buchinit,6,0},
-  {"buchinitfu",4,(void *)buchinitfu,6,0},
-  {"buchreal",5,(void *)buchreal,4,0},
+  {"buchgen",3,(void *)buchgen,6,0},
+  {"buchgenfu",3,(void *)buchgenfu,6,0},
+  {"buchimag",3,(void *)buchimag,4,0},
+  {"buchinit",3,(void *)buchinit,6,0},
+  {"buchinitfu",3,(void *)buchinitfu,6,0},
+  {"buchreal",4,(void *)buchreal,4,0},
   {"bytesize",10,(void *)taille2,2,0},
   {"ceil",1,(void *)gceil,2,0},
   {"centerlift",1,(void *)centerlift,2,0},
@@ -119,6 +120,7 @@ entree fonctions[]={
   {"coeff",21,(void *)truecoeff,2,0},
   {"compimag",2,(void *)compimag,4,0},
   {"compo",21,(void *)compo,2,0},
+  {"compositum",2,(void *)compositum,6,0},
   {"comprealraw",2,(void *)comprealraw,4,0},
   {"concat",2,(void *)concat,8,0},
   {"conj",1,(void *)gconj,2,0},
@@ -140,6 +142,7 @@ entree fonctions[]={
   {"dilog",1,(void *)dilog,3,0},
   {"disc",1,(void *)discsr,7,0},
   {"discf",1,(void *)discf,6,0},
+  {"discf2",1,(void *)discf2,6,0},
   {"divisors",1,(void *)divisors,4,0},
   {"divres",2,(void *)gdiventres,1,0},
   {"divsum",22,(void *)divsomme,9,0},
@@ -189,6 +192,7 @@ entree fonctions[]={
   {"hermite",1,(void *)hnf,8,0},
   {"hermitehavas",1,(void *)hnfhavas,8,0},
   {"hermitemod",2,(void *)hnfmod,8,0},
+  {"hermitemodid",2,(void *)hnfmodid,8,0},
   {"hess",1,(void *)hess,8,0},
   {"hilb",30,(void *) hil,4,0},
   {"hilbert",11,(void *)hilb,8,0},
@@ -415,6 +419,7 @@ entree fonctions[]={
   {"sigmak",24,(void *)sumdivk,4,0},
   {"sign",10,(void *)gsigne,1,0},
   {"signat",1,(void *)signat,8,0},
+  {"signunit",1,(void *)signunit,6,0},
   {"simplefactmod",2,(void *)simplefactmod,4,0},
   {"simplify",1,(void *)simplify,2,0},
   {"sin",1,(void *)gsin,3,0},
