@@ -168,7 +168,7 @@ GEN auxdecomp(GEN n, long all)
   byteptr d=diffptr;
   unsigned char c;
   GEN p,z,p1,p2,f;
-  long nb=0,j,k,lim,av1,av2,av3,av4,sn;
+  long nb=0,nbx=0,j,k,lim,av1,av2,av3,av4,sn;
   
   if (typ(n)!=1) err(arither1);
   if (!(sn=signe(n))) err(arither2);
@@ -201,7 +201,7 @@ GEN auxdecomp(GEN n, long all)
 		  f=n;
 		  if(all==1)
 		    while (!((cmpii(mulii(p,p),f)>0)||pseudoprime(f))) 
-		      f=ellfacteur(f);
+		      {f=ellfacteur(f);nbx=1;}
 		  av4=avma;
 		  f=gerepile(av3,av4,gcopy(f));
 		  nb++;
@@ -218,7 +218,7 @@ GEN auxdecomp(GEN n, long all)
 		  f=n;
 		  if(all==1)
 		    while (!((cmpii(mulii(p,p),f)>0)||pseudoprime(f))) 
-		      f=ellfacteur(f);
+		      {f=ellfacteur(f);nbx=1;}
 		  av4=avma;
 		  f=gerepile(av3,av4,gcopy(f));
 		  nb++;
@@ -241,6 +241,7 @@ GEN auxdecomp(GEN n, long all)
       p1[j]=(long)p;
       p+=lg(p);
     }
+  if((DEBUGLEVEL>3)&&nbx) {fprintf(errfile,"\n");fflush(stdout);}
   return z;
 }
 
@@ -615,7 +616,7 @@ GEN ellfacteur(GEN n1)
   static long i,k,m,av;
   static GEN glob;
   taillef=lgef(n1);
-  nombre=10*lgef(n1);
+  nombre=5*(lgef(n1)-2);
   global=cgeti(taillef);
   av=avma;
   n=absi(n1);
@@ -631,8 +632,8 @@ GEN ellfacteur(GEN n1)
     if (pur(m,&k))
       {
         for(i=nombre;i;i--) {affii((GEN)x2[i],(GEN)x1[i]);affii((GEN)y2[i],(GEN)y11[i]);}
-	if(DEBUGLEVEL>3) {printf(" x");fflush(stdout);}
-        if(glob = ellmult(k))
+	if(DEBUGLEVEL>3) {fprintf(errfile,".");fflush(stdout);}
+        if((glob = ellmult(k))&&(cmpii(glob,n)))
           {
             if (cmpii(mulii(glob,glob),n)>0) diviiz(n,glob,global);
             else affii(glob,global);
