@@ -8,6 +8,7 @@
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
 # include "genpari.h"
+GEN differente(GEN nf);
 
 #define coef1(a,i,j)      (*((long*)(*(a+(j)+1))+(i)+1))
 #define gcoef1(a,i,j)     (GEN)coef1(a,i,j)
@@ -145,6 +146,15 @@ GEN discf(GEN x)
   return gerepile(av,tetpil,gcopy(y));
 }
 
+GEN discf2(GEN x)
+{
+  GEN y;
+  long av,tetpil;
+
+  av=avma;allbase(x,0,&y);tetpil=avma;
+  return gerepile(av,tetpil,gcopy(y));
+}
+
 GEN smalldiscf(GEN x)
 {
   GEN y;
@@ -629,9 +639,9 @@ GEN cleanmod(GEN x,GEN detmat,GEN detmatsur2)
   return y;
 }
 
-GEN hnfmod(GEN x,GEN detmat)
+GEN allhnfmod(GEN x,GEN detmat,long all)
 {
-  long li,co,av,tetpil,i,j,def,ldef,lim;
+  long li,co,av,tetpil,i,j,jm1,def,ldef,lim;
   GEN b,q,w,p1,y,d,u,v,dms2;
 
   if(typ(x)!=19) err(hnfer1);
@@ -642,42 +652,30 @@ GEN hnfmod(GEN x,GEN detmat)
   for(i=li-1;i>=ldef;i--)
     {
       def--;j=def-1;while(j&&(!signe(gcoeff(y,i,j)))) j--;
-      while(j>1)
+      while(j)
 	{
-	  d=bezout(gcoeff(y,i,j),gcoeff(y,i,j-1),&u,&v);
+	  jm1=j-1;if(!jm1) jm1=def;
+	  d=bezout(gcoeff(y,i,j),gcoeff(y,i,jm1),&u,&v);
 	  if(signe(u))
 	    {
-	      if(signe(v)) p1=gadd(gmul(u,(GEN)y[j]),gmul(v,(GEN)y[j-1]));
+	      if(signe(v)) p1=gadd(gmul(u,(GEN)y[j]),gmul(v,(GEN)y[jm1]));
 	      else p1=gmul(u,(GEN)y[j]);
 	    }
-	  else p1=gmul(v,(GEN)y[j-1]);
-	  y[j]=lsub(gmul(divii(gcoeff(y,i,j),d),(GEN)y[j-1]),gmul(divii(gcoeff(y,i,j-1),d),(GEN)y[j]));
+	  else p1=gmul(v,(GEN)y[jm1]);
+	  y[j]=lsub(gmul(divii(gcoeff(y,i,j),d),(GEN)y[jm1]),gmul(divii(gcoeff(y,i,jm1),d),(GEN)y[j]));
 	  y[j]=(long)cleanmod((GEN)y[j],detmat,dms2);
-	  y[j-1]=(long)cleanmod(p1,detmat,dms2);
+	  y[jm1]=(long)cleanmod(p1,detmat,dms2);
 	  j--;while(j&&(!signe(gcoeff(y,i,j)))) j--;
 	  if(avma<lim) {tetpil=avma;y=gerepile(av,tetpil,gcopy(y));}
 	}
-      if(j==1)
-	{
-	  d=bezout(gcoeff(y,i,1),gcoeff(y,i,def),&u,&v);
-	  if(signe(u))
-	    {
-	      if(signe(v)) p1=gadd(gmul(u,(GEN)y[1]),gmul(v,(GEN)y[def]));
-	      else p1=gmul(u,(GEN)y[1]);
-	    }
-	  else p1=gmul(v,(GEN)y[def]);
-	  y[1]=lsub(gmul(divii(gcoeff(y,i,1),d),(GEN)y[def]),gmul(divii(gcoeff(y,i,def),d),(GEN)y[1]));
-	  y[1]=(long)cleanmod((GEN)y[1],detmat,dms2);
-	  y[def]=(long)cleanmod(p1,detmat,dms2);
-	}
-      if(avma<lim) {tetpil=avma;y=gerepile(av,tetpil,gcopy(y));}
     }
-  b=detmat;w=cgetg(li,19);def--;
+  b=detmat;
+  w=cgetg(li,19);def--;
   for(i=li-1;i>=1;i--)
     {
       d=bezout(gcoeff(y,i,i+def),b,&u,&v);w[i]=lmod(gmul(u,(GEN)y[i+def]),b);
       if(!signe(gcoeff(w,i,i))) coeff(w,i,i)=(long)d;
-      if(i>1) b=divii(b,d);
+      if((i>1)&&all) b=divii(b,d);
     }
   for(i=li-2;i>=1;i--)
     {
@@ -690,6 +688,16 @@ GEN hnfmod(GEN x,GEN detmat)
   tetpil=avma;return gerepile(av,tetpil,gcopy(w));
 }
 
+GEN hnfmod(GEN x, GEN detmat)
+{
+  return allhnfmod(x,detmat,1);
+}
+
+GEN hnfmodid(GEN x, GEN p)
+{
+  return allhnfmod(x,p,0);
+}
+
 GEN hnfhavas(GEN x)
  /* rend [y,U,V] tel que y=V.x.U , V vecteur de permutation, U matrice 
     unimodulaire, y forme HNF de x */
@@ -1331,9 +1339,10 @@ GEN galois(GEN x, long prec)
 
 GEN initalg(GEN x, long prec)
 {
-  GEN y,p1,p2,p3,p4,p5,p6,p7,p10,p11,fieldd,polr,ptrace,dx,adx,polmax,s,a,dxn,adxn,sn,phimax,ind;
-  long tx=typ(x),n=lgef(x)-3,i,j,av=avma,av1,av2,av3,v,k,r1,r2,ru,imax,numb,flc,tetpil;
+  GEN y,p1,p2,p3,p4,p5,p6,p7,p10,p11,p12,fieldd,polr,ptrace,dx,adx,polmax,s,a,dxn,adxn,sn,phimax,ind;
+  long tx=typ(x),n=lgef(x)-3,i,j,av=avma,av1,av2,av3,v,k,j1,j2,lgp,r1,r2,ru,imax,numb,flc,tetpil;
 
+  if(DEBUGLEVEL) timer2();
   if((tx!=10)||(n<=0)) err(poltyper);
   p1=content(x);p1=gcmp1(p1) ? x: gdiv(x,content(x));
   for(k=2;k<=n+2;k++) if(typ((GEN)p1[k])!=1) err(impl,"general algebraic extension");
@@ -1348,8 +1357,10 @@ GEN initalg(GEN x, long prec)
 	}
     }
   p1=factor(x);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps factpol: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   if(lgef(gcoeff(p1,1,1))!=n+3) err(redper1);
   r1=sturm(x);p4=allbase4(x,0,&fieldd);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps round4: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   if(r1<n)
     {
       polr=roots(x,prec);p3=cgetg(n+1,19);
@@ -1387,6 +1398,7 @@ GEN initalg(GEN x, long prec)
       p1=lllgramint(p2);
       s=(n>1)?gsub(gsqr((GEN)x[n+1]),gmul2n((GEN)x[n],1)):gsqr((GEN)x[2]);
     }
+  if(DEBUGLEVEL) {fprintf(errfile,"temps matrice T2: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   v=varn(x);dx=discsr(x);adx=absi(dx);polmax=x;imax=0;
   a=cgetg(n+1,18);for(i=1;i<=n;i++) a[i]=lmul(p4,(GEN)p1[i]);
   for(numb=0,i=1;i<=n;i++)
@@ -1426,6 +1438,7 @@ GEN initalg(GEN x, long prec)
       for(;j>=2;j-=2) setsigne((GEN)polmax[j],-signe((GEN)polmax[j]));
       phimax=gneg(phimax);
     }
+  if(DEBUGLEVEL) {fprintf(errfile,"temps polmax: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   p2=lift(gsubst(p4,v,polymodrecip(gmodulcp(phimax,x))));
   p3=cgetg(n+1,19);
   for(j=1;j<=n;j++)
@@ -1433,8 +1446,8 @@ GEN initalg(GEN x, long prec)
       p4=cgetg(n+1,18);p3[j]=(long)p4;
       for(i=1;i<=n;i++) p4[i]=(long)truecoeff((GEN)p2[j],i-1);
     }
-  p4=denom(p3);
-  p2=gdiv(hnf(gmul(p4,p3)),p4);p3=cgetg(n+1,17);
+  p4=denom(p3);p1=gmul(p4,p3);
+  p2=gdiv(hnfmod(p1,detint(p1)),p4);p3=cgetg(n+1,17);
   for(j=1;j<=n;j++) 
     {
       p1=gzero;for(i=n;i;i--) p1=gadd(gcoeff(p2,i,j),gmul(p1,polx[v]));
@@ -1450,14 +1463,19 @@ GEN initalg(GEN x, long prec)
   p5=cgetg(n*n+1,19);
   for(j=1;j<=n*n;j++) p5[j]=lgetg(n+1,18);
   p11=cgetg(n+1,19);for(j=1;j<=n;j++) p11[j]=lgetg(n+1,18);
-  for(j=1;j<=n*n;j++) 
-    {
-      p10=gmod(gmul((GEN)p3[(j-1)%n+1],(GEN)p3[((j-1)/n)+1]),polmax);      
-      for(i=1;i<=n;i++) 
-	coeff(p5,i,j)=(long)truecoeff(p10,i-1);
-      coeff(p11,(j-1)%n+1,((j-1)/n)+1)=(long)trace(gmodulcp(p10,polmax));
-    }
+  p12=polsym(polmax,lgef(polmax)-4);
+  for(i=1;i<=n;i++)
+    for(j=i;j<=n;j++)
+      {
+	p10=gmod(gmul((GEN)p3[j],(GEN)p3[i]),polmax);
+	j1=j+(i-1)*n;j2=i+(j-1)*n;lgp=lgef(p10)-2;
+	for(k=1;k<=lgp;k++) coeff(p5,k,j1)=coeff(p5,k,j2)=p10[k+1];
+	for(;k<=n;k++) coeff(p5,k,j1)=coeff(p5,k,j2)=zero;
+	coeff(p11,i,j)=coeff(p11,j,i)=(long)trace9(p10,p12);
+      }
+  if(DEBUGLEVEL) {fprintf(errfile,"temps table de mult: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   p2=roots(polmax,prec);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps racines: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   for(i=1;i<=r1;i++) p2[i]=lreal((GEN)p2[i]);
   tetpil=avma;y=cgetg(10,17);y[1]=lcopy(polmax);y[8]=(long)ginv(p1);
   y[9]=lmul((GEN)y[8],p5);p1=cgetg(3,17);
@@ -1482,21 +1500,26 @@ GEN initalg(GEN x, long prec)
   av3=avma;
   y[5]=lpile(av2,av3,gcopy(p7));p4=cgetg(ru+1,17);y[6]=(long)p4;
   for(i=1;i<=ru;i++) p4[i]=lcopy((GEN)p2[max(i,(i<<1)-r1)]);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps matrices: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   y[7]=lcopy(p3);((GEN)y[5])[5]=(long)differente(y);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps fin differente: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   return gerepile(av,tetpil,y);
 }
 
-GEN initalg0(GEN x, long prec)
+GEN initalgall0(GEN x, long fldif, long prec)
 {
-  GEN y,p1,p2,p3,p4,p5,p6,p7,p10,p11,fieldd,dx,ind;
-  long tx=typ(x),n=lgef(x)-3,i,j,av=avma,av2,av3,v,k,r1,r2,ru,tetpil;
+  GEN y,p1,p2,p3,p4,p5,p6,p7,p10,p11,p12,fieldd,dx,ind;
+  long tx=typ(x),n=lgef(x)-3,i,j,av=avma,av2,av3,v,k,r1,r2,ru,tetpil,j1,j2,lgp;
 
+  if(DEBUGLEVEL) timer2();
   if((tx!=10)||(n<=0)) err(poltyper);
   for(k=2;k<=n+2;k++) if(typ((GEN)x[k])!=1) err(initer1);
   if(!gcmp1((GEN)x[n+2])) err(initer2);
   p1=factor(x);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps factpol: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   if(lgef(gcoeff(p1,1,1))!=n+3) err(redper1);
   r1=sturm(x);p3=allbase4(x,0,&fieldd);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps round4: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   v=varn(x);dx=discsr(x);
   if(!carrecomplet(divii(dx,fieldd),&ind)) err(initalgbug1);
   p1=cgetg(n+1,19);
@@ -1508,14 +1531,19 @@ GEN initalg0(GEN x, long prec)
   p5=cgetg(n*n+1,19);
   for(j=1;j<=n*n;j++) p5[j]=lgetg(n+1,18);
   p11=cgetg(n+1,19);for(j=1;j<=n;j++) p11[j]=lgetg(n+1,18);
-  for(j=1;j<=n*n;j++) 
-    {
-      p10=gmod(gmul((GEN)p3[(j-1)%n+1],(GEN)p3[((j-1)/n)+1]),x);      
-      for(i=1;i<=n;i++) 
-	coeff(p5,i,j)=(long)truecoeff(p10,i-1);
-      coeff(p11,(j-1)%n+1,((j-1)/n)+1)=(long)trace(gmodulcp(p10,x));
-    }
+  p12=polsym(x,lgef(x)-4);
+  for(i=1;i<=n;i++)
+    for(j=i;j<=n;j++)
+      {
+	p10=gmod(gmul((GEN)p3[j],(GEN)p3[i]),x);
+	j1=j+(i-1)*n;j2=i+(j-1)*n;lgp=lgef(p10)-2;
+	for(k=1;k<=lgp;k++) coeff(p5,k,j1)=coeff(p5,k,j2)=p10[k+1];
+	for(;k<=n;k++) coeff(p5,k,j1)=coeff(p5,k,j2)=zero;
+	coeff(p11,i,j)=coeff(p11,j,i)=(long)trace9(p10,p12);
+      }
+  if(DEBUGLEVEL) {fprintf(errfile,"temps table de mult: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   p2=roots(x,prec);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps racines: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   for(i=1;i<=r1;i++) p2[i]=lreal((GEN)p2[i]);
   tetpil=avma;y=cgetg(10,17);y[1]=lcopy(x);y[8]=(long)ginv(p1);
   y[9]=lmul((GEN)y[8],p5);p1=cgetg(3,17);
@@ -1540,10 +1568,16 @@ GEN initalg0(GEN x, long prec)
   av3=avma;
   y[5]=lpile(av2,av3,gcopy(p7));p4=cgetg(ru+1,17);y[6]=(long)p4;
   for(i=1;i<=ru;i++) p4[i]=lcopy((GEN)p2[max(i,(i<<1)-r1)]);
-  y[7]=lcopy(p3);((GEN)y[5])[5]=(long)differente(y);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps matrices: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
+  y[7]=lcopy(p3);((GEN)y[5])[5]=fldif?(long)differente(y):zero;
+  if(DEBUGLEVEL&&fldif) {fprintf(errfile,"temps fin differente: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   return gerepile(av,tetpil,y);
 }
 
+GEN initalg0(GEN x, long prec)
+{
+  return initalgall0(x,1,prec);
+}
 
 GEN initalg2(GEN x, long prec)
 {
@@ -1565,7 +1599,7 @@ GEN initalg2(GEN x, long prec)
     }
   p1=factor(x);
   if(lgef(gcoeff(p1,1,1))!=n+3) err(redper1);
-  r1=sturm(x);p4=allbase4(x,0,&fieldd);
+  r1=sturm(x);p4=allbase(x,0,&fieldd);
   if(r1<n)
     {
       polr=roots(x,prec);p3=cgetg(n+1,19);
@@ -1656,7 +1690,7 @@ GEN galoisconj2(GEN x, long prec)
   n=lgef(x)-3;if(n<=0) err(galer1);
   v=varn(x);p1=factor(x);
   if((lg((GEN)p1[1])>2)||(!gcmp1(gcoeff(p1,1,2)))) err(galcer1);
-  polr=roots(x,prec);p1=(GEN)polr[1];b=allbase4(x,0,&di);
+  polr=roots(x,prec);p1=(GEN)polr[1];b=allbase(x,0,&di);
   w=cgetg(n+1,17);w[1]=un;for(i=2;i<=n;i++) w[i]=(long)gsubst((GEN)b[i],v,p1);
   y=cgetg(n+1,17);y[1]=(long)polx[v];
   for(i=2;i<=n;i++)
@@ -1700,3 +1734,57 @@ GEN galoisconj(GEN x, long prec)
     }
   tetpil=avma;return gerepile(av,tetpil,gcopy(y));
 }
+
+
+/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
+/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
+/*~									~*/
+/*~		      COMPOSITUM OF TWO NUMBER FIELDS                   ~*/
+/*~									~*/
+/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
+/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
+
+#define sign2(x) (x>=0?0:1)
+
+GEN compositum(GEN pol1, GEN pol2)
+{
+  long av=avma,tetpil,i,j,a,b,l,l1,l2,fl,c,degmax,a1;
+  GEN m,sol,pro0,pro1,pro2,pro3,k1,p;
+
+  if((typ(pol1)!=10)||(typ(pol2)!=10)) err(talker,"incorrect type in compositum");
+  if((varn(pol1)!=0)||(varn(pol2)!=0)) err(talker,"not the x variable in compositum");
+  l1=lgef(pol1)-3;l2=lgef(pol2)-3;
+  if((l1<=0)||(l2<=0)) err(talker,"constant polynomial in compositum");
+  l=l1*l2;m=cgetg(l+2,19);
+  for(j=1;j<=l+1;j++) m[j]=lgetg(l+1,18);
+  fl=1;sol=polun[0];degmax=0;
+  for(c=1,a=1;(c<=l)&&fl;a=sign2(a)-a)
+    {
+      pro0=cgetg(3,10);pro0[1]=evallgef(3)+evalsigne(1);pro0[2]=(long)polun[MAXVARN];
+      pro1=gadd(polx[0],gmulsg(a,polx[MAXVARN]));
+      for(b=0;b<=l;b++)
+	{
+	  for(i=0;i<l1;i++)
+	    {
+	      pro3=truecoeff(pro0,i);setvarn(pro3,0);
+	      pro2=gmod(pro3,pol2);
+	      for(j=0;j<l2;j++)	coeff(m,i*l2+j+1,l+1-b)=(long)truecoeff(pro2,j);
+	    }
+	  if(b<l) pro0=gmod(gmul(pro0,pro1),pol1);
+	}
+      k1=kerint(m);c++;
+      for(a1=1;(a1<lg(k1))&&fl;a1++)
+	{
+	  p=gtopoly((GEN)k1[a1],0);
+	  pro0=(GEN)factor(p)[1];
+	  for(i=1;(i<lg(pro0))&&fl;i++) 
+	    if(lgef((GEN)pro0[i])>lgef(sol)) 
+	      {
+		sol=(GEN)pro0[i];degmax=lgef(sol)-3;fl=(degmax<l);
+	      }
+	}
+    }
+  tetpil=avma;return gerepile(av,tetpil,gdiv(sol,(GEN)sol[lgef(sol)-1]));
+}
+
+	  
