@@ -11,10 +11,12 @@
 #define TRUE 1
 #define FALSE 0
 
+FILE *temp;
 GEN pradical(GEN nf,GEN p),pol_min(GEN alpha,GEN nf,GEN p,GEN algebre,GEN algebre1);
 GEN eval_pol(GEN nf,GEN pol,GEN alpha,GEN p,GEN algebre,GEN algebre1);
 GEN lens(GEN nf,GEN p,GEN a),two_elt(GEN nf,GEN p,GEN ideal);
 GEN two_elt(GEN nf, GEN p, GEN ideal),fasthnf(GEN x, GEN detmat),differente(GEN nf);
+GEN element_mulid(GEN nf, GEN x, long i);
 long idealvalint(GEN nf, GEN x, GEN vp);
 
 GEN maxord(GEN p,GEN f,long mf);
@@ -29,9 +31,10 @@ GEN testb(GEN p,GEN fa,long Da,GEN theta,long Dt);
 GEN testc(GEN p, GEN fa, long c, GEN alph2, long Ma, GEN thet2, long Mt);
 GEN testd(GEN p,GEN fa,long c,long Da,GEN alph2,long Ma,GEN theta);
 GEN csrch(GEN p,GEN fa,GEN gamma);
+GEN dedek(GEN f,GEN p,GEN g);
 long cbezout(long a,long b,long *u,long *v);
 long clcm(long a,long b);
-long dedek(GEN f,GEN p,GEN w);
+
 
 /*******************************************************************
 
@@ -93,17 +96,19 @@ GEN primedec(GEN nf, GEN p)
 l'ordre croissant des degres residuels */
 
 {
-  long av=avma,tetpil,i,j,k,kbar,l3,np,c,i1,i2,indice,N;
+  long av=avma,tetpil,i,j,k,v,kbar,l3,np,c,i1,i2,indice,N;
   GEN f,list,list2,ip,elementh,hensemble;
-  GEN algebre,algebre1,b,b2,mat1,mat2,identmodp;
-  GEN alpha,beta1,beta2,p1,p2,p3,unmodp,zmodp,vecteur,pol;
-  GEN T=(GEN)nf[1],pidmat;
+  GEN algebre,algebre1,b,b2,mat1,mat2;
+  GEN alpha,beta1,p1,p2,p3,p4,unmodp,zmodp,vecteur,pol;
+  GEN T=(GEN)nf[1],pidmat,ppuin;
 
+  if(DEBUGLEVEL>=3) timer2();
   if((typ(nf)!=17)||(lg(nf)<10)) err(idealer1);
   N=lgef(T)-3;
   if(signe(modii((GEN)nf[4],p)))
     {
       f=centerlift(factmod(T,p));np=lg((GEN)f[1]);
+      if(DEBUGLEVEL>=3) {fprintf(errfile,"temps factmod: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
       list=cgetg(np,17);
       for(i=1;i<np;i++)
 	{
@@ -114,10 +119,11 @@ l'ordre croissant des degres residuels */
 	    {
 	      p1[2]=(long)p2;p1[3]=un;p1[4]=lstoi(N);
 	      p3=cgetg(N+1,18);p1[5]=(long)p3;p3[1]=un;
-	      p2[1]=zero;for(j=2;j<=N;j++) p3[j]=p2[j]=zero;
+	      p2[1]=(long)p;for(j=2;j<=N;j++) p3[j]=p2[j]=zero;
 	    }
 	  else
 	    {
+	      v=ggval(subres(p3,T),p)/(l3-2);if(v>1) p3[2]=ladd((GEN)p3[2],p);
 	      for(j=1;j<l3;j++) p2[j]=p3[j+1];
 	      for(j=l3;j<=N;j++) p2[j]=zero;
 	      p1[2]=lmul((GEN)nf[8],p2);
@@ -125,19 +131,22 @@ l'ordre croissant des degres residuels */
 	      p1[4]=lstoi(l3-2);p1[5]=(long)lens(nf,p,(GEN)p1[2]);
 	    }
 	}
+      if(DEBUGLEVEL>=3) {fprintf(errfile,"temps lens etc...: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}      
       p1=stoi(4);tetpil=avma;return gerepile(av,tetpil,vecsort(list,p1));
     }
   else 
     {
       list=cgetg(N+1,17);for(i=1;i<=N;i++) list[i]=lgetg(6,17);
       ip=pradical(nf,p);
+      if(DEBUGLEVEL>=3) {fprintf(errfile,"temps pradical: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
       unmodp=gmodulcp(gun,p);zmodp=gmodulcp(gzero,p);
-      identmodp=gmul(unmodp,idmat(N));pidmat=gscalmat(p,N);
-      c=0;indice=0;
-      vecteur=(GEN)(identmodp[1]);
+      pidmat=cgetg(N+1,18);vecteur=cgetg(N+1,18);
+      for(i1=2;i1<=N;i1++) {pidmat[i1]=zero;vecteur[i1]=(long)zmodp;}
+      pidmat[1]=(long)p;vecteur[1]=(long)unmodp;
+      ppuin=gpuigs(p,N);c=0;indice=0;
       hensemble=cgetg(N+1,17); 
       if(lg(ip)<N+1) {c=1;hensemble[1]=(long)ip;}
-      while(c!=0)
+      while(c)
         {  
            elementh=(GEN)(hensemble[c]);k=lg(elementh)-1;kbar=N-k;
            algebre=gmul(unmodp,suppl(concat(elementh,vecteur)));
@@ -152,29 +161,24 @@ l'ordre croissant des degres residuels */
            for(i1=1;i1<=kbar;i1++)
              for(i2=1;i2<=kbar;i2++)
                coeff(mat1,i2,i1)=coeff(b2,k+i2,i1);
+	   mat2=cgetg(k+N+1,19);
+	   for(i1=1;i1<=k;i1++) mat2[i1]=elementh[i1];
            mat1=gmul(unmodp,ker(mat1));
            if(lg(mat1)>2)
              {
                 alpha=gmul(algebre1,(GEN)mat1[2]);
                 pol=pol_min(alpha,nf,p,algebre,algebre1);
                 setvarn(pol,0);
-                for(i1=0;!gegal(gsubst(pol,0,stoi(i1)),zmodp);i1++); 
-                p1=gmul(unmodp,gaddgs(polx[0],-i1));p2=gdiv(pol,p1);
-                beta1=eval_pol(nf,p1,alpha,p,algebre,algebre1);
-                beta2=eval_pol(nf,p2,alpha,p,algebre,algebre1);
-                mat1=cgetg(k+N+1,19);mat2=cgetg(k+N+1,19);
-                for(i1=1;i1<=k;i1++)
-                  {
-                     mat1[i1]=elementh[i1];
-                     mat2[i1]=elementh[i1];
-                  }
-                for(i1=1;i1<=N;i1++)
-                  {
-                     mat1[k+i1]=(long)(element_mul(nf,(GEN)identmodp[i1],beta1));
-                     mat2[k+i1]=(long)(element_mul(nf,(GEN)identmodp[i1],beta2));
-                  }
-                hensemble[c]=(long)(image(mat1));
-                c++;hensemble[c]=(long)(image(mat2));
+		p1=(GEN)factmod(pol,p)[1];
+		for(i=1;i<lg(p1);i++)
+		  {
+		    beta1=eval_pol(nf,(GEN)p1[i],alpha,p,algebre,algebre1);
+		    for(i1=1;i1<=N;i1++)
+		      mat2[k+i1]=(long)element_mulid(nf,beta1,i1);
+		    hensemble[c]=(long)(image(mat2));c++;
+		  }
+		c--;
+		if(DEBUGLEVEL>=3) {fprintf(errfile,"temps hensemble[%ld]: ",c);fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
              }
            else
              {
@@ -182,8 +186,9 @@ l'ordre croissant des degres residuels */
                 p1[1]=(long)p;p1[4]=lstoi(kbar);
                 p1[2]=(long)two_elt(nf,p,elementh);
                 p1[5]=(long)lens(nf,p,(GEN)p1[2]);
-		p1[3]=lstoi(idealvalint(nf,pidmat,p1));
+		p1[3]=lstoi(elementval2(nf,pidmat,ppuin,p1));
                 c--;
+		if(DEBUGLEVEL>=3) {fprintf(errfile,"temps hensemble[%ld]: ",c);fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
 	     }
         }
       list2=cgetg(indice+1,17);
@@ -367,6 +372,31 @@ GEN element_pow(GEN nf, GEN x, GEN k)
   tetpil=avma;return gerepile(av,tetpil,gcopy(y));
 }
 
+GEN element_mulid(GEN nf, GEN x, long i)
+                
+/* Recoit un vecteur x de longueur N representant x dans la base d'entiers
+(peut etre modulo p) et ressort x.w_i sous forme d'un
+ vecteur */
+
+{
+  long av=avma,tetpil,j,k,N=lgef((GEN)nf[1])-3;
+  GEN s,v,c,p1;
+
+  v=cgetg(N+1,18);
+  for(k=1;k<=N;k++)
+    {
+      s=gzero;
+      for(j=1;j<=N;j++)
+	{
+	  c=gcoeff((GEN)nf[9],k,(i-1)*N+j);p1=(GEN)x[j];
+	  if((signe(c))&&(!gcmp0(p1)))
+	    s=gcmp1(c)?gadd(s,p1):gadd(s,gmul(p1,c));
+	}
+      v[k]=(long)s;
+    }
+  tetpil=avma;return gerepile(av,tetpil,gcopy(v));
+}
+
 GEN pradical(GEN nf, GEN p)
               
 /* Calcule une F_p base du p-radical de Z_K,i.e une
@@ -420,16 +450,25 @@ GEN eval_pol(GEN nf, GEN pol, GEN alpha, GEN p, GEN algebre, GEN algebre1)
 /* Evalue le polynome pol en alpha,element de nf */
 
 {
-   long av=avma,tetpil,i,N=lgef((GEN)nf[1])-3,k=N-lg(algebre1)+1;
+   long av=avma,tetpil,i;
+   long N=lgef((GEN)nf[1])-3,k=N-lg(algebre1)+1,lx=lgef(pol)-3;
    GEN res,valeur,unmodp,zmodp;
 
    res=cgetg(N+1,18);
    unmodp=gmodulcp(gun,p);zmodp=gmul(unmodp,gzero);
    for(i=2;i<=N;i++) res[i]=(long)zmodp;
+/*
    res[1]=(long)unmodp;
    res=gmul(res,(GEN)pol[2]);
    for(i=1;i<=lgef(pol)-3;i++)
      res=gadd(res,gmul((GEN)pol[i+2],element_pow(nf,alpha,stoi(i))));
+  la methode de Horner est beaucoup plus efficace!!!!:
+*/
+   res[1]=lmul(unmodp,(GEN)pol[lx+2]);
+   for(i=lx+1;i>=2;i--)
+     {
+       res=element_mul(nf,alpha,res);res[1]=ladd((GEN)res[1],(GEN)pol[i]);
+     }
    res=inverseimage(algebre,res);
    valeur=cgetg(N-k+1,18);
    for(i=1;i<=N-k;i++) valeur[i]=res[k+i];
@@ -529,12 +568,10 @@ vecteur sur la base d'entiers */
 
 {
    long av=avma,tetpil,N=lgef((GEN)nf[1])-3,j;
-   GEN id,mat,unmodp,y;
+   GEN mat;
 
-   id=idmat(N);unmodp=gmodulcp(gun,p);
-   mat=cgetg(N+1,19);for(j=1;j<=N;j++) mat[j]=(long)element_muli(nf,(GEN)id[j],a);
-   tetpil=avma;y=kerlens(mat,p);
-   return gerepile(av,tetpil,y);
+   mat=cgetg(N+1,19);for(j=1;j<=N;j++) mat[j]=(long)element_mulid(nf,a,j);
+   tetpil=avma;return gerepile(av,tetpil,kerlens(mat,p));
 }
 
 GEN two_elt(GEN nf, GEN p, GEN ideal)
@@ -611,7 +648,7 @@ archimedienne et ressort leur produit sans le reduire */
     {
       dp=mulii(det(x),det(y));tetpil=avma;y=hnfmod(m,dp);
     }
-  else {tetpil=avma;y=hnf(m);}
+  else {dp=detint(m);tetpil=avma;y=gcmp0(dp)?hnf(m):hnfmod(m,dp);}
   if(!gcmp1(dz)) {tetpil=avma;y=gdiv(y,dz);}
   if(f) 
     {
@@ -693,6 +730,21 @@ primedec et ressort leur produit */
   return gerepile(av,tetpil,y);
 }
 
+GEN idealmulspec(GEN nf, GEN x, GEN spec)
+/* a usage interne, aucune verif. ix est un ideal entier sans partie 
+archimedienne en HNF, spec=[a,alpha,n] representant un ideal aZ_K+alpha Z_K
+de norme n. Calcule le produit */
+{
+  long av=avma,tetpil,i,N=lgef((GEN)nf[1])-3;
+  GEN dx,m;
+
+  dx=(GEN)spec[3];for(i=1;i<=N;i++) dx=mulii(dx,gcoeff(x,i,i));
+  m=cgetg((N<<1)+1,19);
+  for(i=1;i<=N;i++) m[i]=(long)element_muli(nf,(GEN)spec[2],(GEN)x[i]);
+  for(i=N+1;i<=(N<<1);i++) m[i]=lmul((GEN)spec[1],(GEN)x[i-N]);
+  tetpil=avma;return gerepile(av,tetpil,fasthnf(m,dx));
+}
+
 long idealvalint(GEN nf, GEN x, GEN vp)
                   
 /* recoit un ideal entier x et un ideal premier vp dans le format
@@ -731,6 +783,73 @@ donne par primedec et calcule la valuation de ix en vp. A usage interne, pas de
   avma=av;return w;
 }
 
+long elementval(GEN nf, GEN x, GEN vp)
+/* a usage interne, pas de verifs. */
+{
+  long av=avma,tetpil,N=lgef((GEN)nf[1])-3,w,bo,i,vd,v,e=itos((GEN)vp[3]);
+  GEN denx,p1,p=(GEN)vp[1],bp,d,r;
+
+  switch(typ(x))
+    {
+    case 1: case 4: case 5:
+      return ggval(x,p)*e;
+    case 10: d=gabs(subres(x,(GEN)nf[1]));
+      x=(GEN)principalideal(nf,x)[1];break;
+    case 9: d=gabs(subres((GEN)x[2],(GEN)nf[1]));
+      x=(GEN)principalideal(nf,x)[1];break;
+    default: err(talker,"incorrect type in elementval");
+    }
+  denx=denom(x);if(!gcmp1(denx)) x=gmul(denx,x);
+  vd=ggval(denx,p);v=ggval(d,p)+N*vd;
+  if(!v) return -vd*e;
+  bo=0;w=0;bp=(GEN)vp[5];
+  do
+    {
+      x=element_muli(nf,x,bp);
+      if(divise((GEN)x[N],p))
+	{
+	  for(i=1;i<=N;i++)
+	    {
+	      p1=dvmdii((GEN)x[i],p,&r);
+	      if(signe(r)) goto labelelv; else x[i]=(long)p1;
+	    }
+	  w++;
+	}
+      else bo=1;
+    }
+  while((bo==0)&&(w<v));
+ labelelv:
+  avma=av;return w-vd*e;
+}
+
+long elementval2(GEN nf, GEN x, GEN d, GEN vp)
+/* a usage interne, pas de verifs. */
+{
+  long av=avma,tetpil,N=lgef((GEN)nf[1])-3,w,bo,i,v;
+  GEN denx,p1,p=(GEN)vp[1],bp,r;
+
+  v=ggval(d,p);
+  if(!v) return 0;
+  bo=0;w=0;bp=(GEN)vp[5];
+  do
+    {
+      x=element_muli(nf,x,bp);
+      if(divise((GEN)x[N],p))
+	{
+	  for(i=1;i<=N;i++)
+	    {
+	      p1=dvmdii((GEN)x[i],p,&r);
+	      if(signe(r)) goto labelelv2; else x[i]=(long)p1;
+	    }
+	  w++;
+	}
+      else bo=1;
+    }
+  while((bo==0)&&(w<v));
+ labelelv2:
+  avma=av;return w;
+}
+
 long idealval(GEN nf, GEN ix, GEN vp)
                   
 /* recoit un ideal ix et un ideal premier vp dans le format
@@ -738,19 +857,21 @@ donne par primedec et calcule la valuation de ix en vp */
 
 {
   long N,v,vd,w,av=avma,i,j,bo;
-  GEN mat,x,d,bp,p=(GEN)vp[1],p1,r,denx;
+  GEN mat,x,d,bp,p,p1,r,denx;
 
   if((typ(nf)!=17)||(lg(nf)<10)) err(idealer1);
   if((typ(vp)!=17)||(lg(vp)!=6)) err(idealer3);
-  if(typ(ix)<=9) ix=principalideal(nf,ix);
+  if(typ(ix)<=9) return elementval(nf,ix,vp);
+  p=(GEN)vp[1];
   if((typ(ix)==17)&&(lg(ix)==3)) x=(GEN)ix[1]; else x=ix;
-  if(typ(x)!=19) err(idealer2);denx=denom(x);
+  if(typ(x)!=19) err(idealer2);
+  denx=denom(x);
   if(!gcmp1(denx)) x=gmul(denx,x);
   N=lgef((GEN)nf[1])-3;if(lg((GEN)x[1])!=(N+1)) err(idealer4);
   if(lg(x)!=(N+1)) x=idealmul(nf,x,idmat(N));
   for(d=gun,i=1;i<=N;i++) d=mulii(d,(GEN)coeff(x,i,i));
-  v=ggval(d,p);if(!v) return 0;
-  vd=ggval(denx,p);
+  v=ggval(d,p);vd=ggval(denx,p);
+  if(!v) return -vd*itos((GEN)vp[3]);
   bo=0;w=0;bp=(GEN)vp[5];
   do
     {
@@ -777,20 +898,22 @@ donne par primedec et calcule la valuation de ix en vp */
   avma=av;return w-vd*itos((GEN)vp[3]);
 }
 
-GEN ideallllred(GEN nf, GEN ix, GEN vdir, long prec)
+GEN ideallllredall(GEN nf, GEN ix, GEN vdir, long prec, long precint)
 {
   long N=lgef((GEN)nf[1])-3,av=avma,tetpil,i,j,f,r1,r2,ru;
-  GEN T,p1,p2,p3,p4,y,alpha,beta,x,x2,r,v,z;
+  GEN T,p1,p2,p3,p4,y,alpha,beta,x,x2,v,z,detmat;
 
   if((typ(nf)!=17)||(lg(nf)<10)) err(idealer1);
   if((!gcmp0(vdir))&&(typ(vdir)!=17)) err(idealer5);
   if(typ(ix)<=9) ix=principalidele(nf,ix);
+  if((typ(ix)==17)&&(lg(ix)==6)) {f=0;ix=idealmulprime(nf,idmat(N),ix);}
   if((typ(ix)==17)&&(lg(ix)==3)) {f=1;x=(GEN)ix[1];} else {f=0;x=ix;}
   if(typ(x)!=19) err(idealer2);
   p1=content(x);if(!gcmp1(p1)) x=gdiv(x,p1);
   p1=(GEN)nf[2];r2=itos((GEN)p1[2]);ru=N-r2;r1=ru-r2;p1=(GEN)nf[5];
   if(!gcmp0(vdir))
     {
+      if(lg(vdir)!=(ru+1)) err(idealer5);
       p3=(GEN)p1[2];p2=cgetg(ru+1,19);
       for(j=1;j<=ru;j++) 
 	{
@@ -803,7 +926,7 @@ GEN ideallllred(GEN nf, GEN ix, GEN vdir, long prec)
     }
   else p1=(GEN)p1[3];
   p4=gmul(x,lllint(x));
-  y=gmul(p4,(GEN)(lllgram(gmul(gtrans(p4),gmul(p1,p4)),2*prec)[1]));
+  y=gmul(p4,(GEN)(lllgram(gmul(gtrans(p4),gmul(p1,p4)),2*precint)[1]));
   for(i=2;(i<=N)&&gcmp0((GEN)y[i]);i++);
   if(i>N) 
     {
@@ -819,18 +942,73 @@ GEN ideallllred(GEN nf, GEN ix, GEN vdir, long prec)
   p1=gmul((GEN)nf[8],p1);p2=cgetg(N+1,19);
   for(i=1;i<=N;i++) p2[i]=(long)element_muli(nf,p1,(GEN)x[i]);
   p1=content(p2);if(!gcmp1(p1)) p2=gdiv(p2,p1);
+  detmat=detint(p2);
   if(f)
     {
-      r=(GEN)nf[6];x2=(GEN)ix[2];v=cgetg(ru+1,17);
+      x2=(GEN)ix[2];v=cgetg(ru+1,17);
       for(i=1;i<=r1;i++) v[i]=(long)glog((GEN)z[i],prec);
       for(i=r1+1;i<=ru;i++) v[i]=lmul2n(glog((GEN)z[i],prec),1);
-      tetpil=avma;y=cgetg(3,17);y[1]=(long)hnf(p2);p1=cgetg(ru+1,17);
+      tetpil=avma;y=cgetg(3,17);y[1]=(long)hnfmod(p2,detmat);p1=cgetg(ru+1,17);
       y[2]=(long)p1;for(i=1;i<=ru;i++) p1[i]=lsub((GEN)x2[i],(GEN)v[i]);
     }
-  else {tetpil=avma;y=hnf(p2);}
+  else {tetpil=avma;y=hnfmod(p2,detmat);}
   return gerepile(av,tetpil,y);
 }
 
+GEN ideallllred(GEN nf, GEN ix, GEN vdir, long prec)
+{
+  return ideallllredall(nf,ix,vdir,prec,prec);
+}
+
+GEN ideallllredpart1(GEN nf, GEN x, GEN vdir, long prec)
+/* a usage interne, pas de verifs */
+{
+  long N=lgef((GEN)nf[1])-3,av=avma,tetpil,i,j,f,r1,r2,ru;
+  GEN T,p1,p2,p3,p4,p5,y,alpha,beta,x2,v,z,detmat;
+
+  p1=content(x);if(!gcmp1(p1)) x=gdiv(x,p1);
+  p1=(GEN)nf[2];r2=itos((GEN)p1[2]);ru=N-r2;p1=(GEN)nf[5];
+  if(!gcmp0(vdir))
+    {
+      p3=(GEN)p1[2];p2=cgetg(ru+1,19);
+      for(j=1;j<=ru;j++) 
+	{
+	  if(typ((GEN)vdir[j])==1) 
+	    p2[j]=lmul2n((GEN)p3[j],itos((GEN)vdir[j])<<1);
+	  else
+	    p2[j]=lmul((GEN)p3[j],gpui(stoi(4),(GEN)vdir[j]));
+	}
+      p1=greal(gmul(p2,(GEN)p1[1]));
+    }
+  else p1=(GEN)p1[3];
+/*
+  p4=gmul(x,lllint(x));
+*/
+  p4=x;
+  p5=lllgram(gmul(gtrans(p4),gmul(p1,p4)),2*prec);
+  y=gmul(p4,(GEN)p5[1]);
+  for(i=2;(i<=N)&&gcmp0((GEN)y[i]);i++);
+  if(i>N) y=gmul(p4,(GEN)p5[2]);
+  T=(GEN)nf[1];alpha=gmul((GEN)nf[7],y);p1=subres(alpha,(GEN)nf[1]);
+  tetpil=avma;z=cgetg(4,17);z[1]=lcopy(x);z[2]=lcopy(y);z[3]=(long)gabs(p1);
+  return gerepile(av,tetpil,z);
+}
+
+GEN ideallllredpart2(GEN nf, GEN arch, GEN gamma, long prec)
+{
+  long N=lgef((GEN)nf[1])-3,av=avma,tetpil,i,r1,r2,ru;
+  GEN p1,r,v,z;
+  
+  z=gmul((GEN)((GEN)(nf[5]))[1],gamma);
+  p1=(GEN)nf[2];r2=itos((GEN)p1[2]);ru=N-r2;r1=ru-r2;
+  v=cgetg(ru+1,17);
+  for(i=1;i<=r1;i++) v[i]=(long)glog((GEN)z[i],prec);
+  for(i=r1+1;i<=ru;i++) v[i]=lmul2n(glog((GEN)z[i],prec),1);
+  tetpil=avma;p1=cgetg(ru+1,17);
+  for(i=1;i<=ru;i++) p1[i]=lsub((GEN)arch[i],(GEN)v[i]);
+  return gerepile(av,tetpil,p1);
+}
+
 GEN minideal(GEN nf, GEN ix, GEN vdir, long prec)
 {
   long N=lgef((GEN)nf[1])-3,av=avma,tetpil,i,j,f,r1,r2,ru;
@@ -866,23 +1044,48 @@ GEN differente(GEN nf)
 /* Calcule la differente de nf */
 
 {
-  long av=avma,tetpil,i,j,k,vi,ei,v,nb_p;
-  GEN ideal,premiers,mat_diff,liste_id,p1;
+  long av=avma,tetpil,i,j,k,vi,ei,v,nb_p,N,vpc,a;
+  GEN ideal,premiers,mat_diff,liste_id,p1,p2,pcon,pr,pol;
 
-  ideal=hnf(gmul((GEN)nf[3],ginv((GEN)((GEN)nf[5])[4])));
+  pol=(GEN)nf[1];N=lgef(pol)-3;
+  if(DEBUGLEVEL) fprintf(errfile,"calcul de la differente\n");
+  if(gcmp1((GEN)nf[4]))
+    {
+      p1=gmodulcp(deriv(pol,varn(pol)),pol);p2=idmat(N);
+      tetpil=avma;return gerepile(av,tetpil,idealmul(nf,p1,p2));
+    }
+  ideal=gmul((GEN)nf[3],ginv((GEN)((GEN)nf[5])[4]));
+  ideal=gdiv(ideal,pcon=content(ideal));
+  if(DEBUGLEVEL) {fprintf(errfile,"temps D*delta^-1: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
+  ideal=hnfmodid(ideal,divii((GEN)nf[3],pcon));
+  if(DEBUGLEVEL) {fprintf(errfile,"temps hnf(D*delta^-1): ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   mat_diff=idmat(lgef((GEN)nf[1])-3);premiers=factor(gabs((GEN)nf[3]));
   nb_p=lg((GEN)premiers[1]);
+  if(DEBUGLEVEL) {fprintf(errfile,"temps factor(D): ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);}
   for(i=1;i<nb_p;i++)
     {
-      liste_id=primedec(nf,gcoeff(premiers,i,1));vi=itos(gcoeff(premiers,i,2));
+      liste_id=primedec(nf,pr=gcoeff(premiers,i,1));vi=itos(gcoeff(premiers,i,2));
+      vpc=ggval(pcon,pr);
       for(j=1;j<lg(liste_id);j++)
 	{
 	  p1=(GEN)liste_id[j];ei=itos((GEN)p1[3]);
 	  if(ei>1)
 	    {
-	      v=ei*vi-idealval(nf,ideal,p1);
-	      for(k=1;k<=v;k++)
-		mat_diff=idealmulprime(nf,mat_diff,p1);
+	      if(DEBUGLEVEL) 
+		{
+		  temp=outfile;outfile=errfile;
+		  fprintf(errfile,"traitement de ");output(p1);
+		  fflush(errfile);outfile=temp;
+		}
+	      if(signe(ressi(ei,pr))) v=ei-1;
+	      else
+		{
+		  v=ei*(vi-vpc)-idealval(nf,ideal,p1);
+		}
+	      a=1+(v-1)/ei;p2=cgetg(4,17);p2[1]=(long)gpuigs(pr,a);
+	      p2[2]=(long)element_pow(nf,(GEN)p1[2],stoi(v));
+	      p2[3]=(long)gpui(pr,mulsi(v,(GEN)p1[4]));
+	      mat_diff=idealmulspec(nf,mat_diff,p2);
 	    }
 	}
     }
@@ -936,7 +1139,7 @@ GEN idealpowprime(GEN nf, GEN vp, GEN n, long prec)
 {
   long N=lgef((GEN)nf[1])-3,RU,av=avma,tetpil,s,i,m,ns;
   unsigned long j;
-  GEN x,p1,p2,p3;
+  GEN x,p1,p2,p3,vpinv;
 
   if((typ(nf)!=17)||(lg(nf)<10)) err(idealer1);
   if((typ(vp)!=17)||(lg(vp)!=6)) err(idealer3);
@@ -952,7 +1155,12 @@ GEN idealpowprime(GEN nf, GEN vp, GEN n, long prec)
     }
   x[2]=(long)p1;
   s=signe(n);if(!s) return x;
-  if(s<0) n=negi(n);
+  if(s<0) 
+    {
+      n=negi(n);vpinv=cgetg(6,17);vpinv[1]=vp[1];vpinv[2]=vp[5];
+      vpinv[3]=vp[3];vpinv[4]=(long)subsi(N,(GEN)vp[4]);vpinv[5]=vp[2];
+      vp=vpinv;
+    }
   if(gcmpgs(n,16)<0)
     {
       ns=n[2];
@@ -982,12 +1190,7 @@ GEN idealpowprime(GEN nf, GEN vp, GEN n, long prec)
 	    }
 	}
     }
-  if(s>0) return gerepile(av,tetpil,x);
-  else 
-    {
-      x=idealinv(nf,x);tetpil=avma;
-      return gerepile(av,tetpil,ideallllred(nf,x,gzero,prec));
-    }
+  return gerepile(av,tetpil,x);
 }
 
 /* Eleve un ideal ix a la puissance n ou n est dans Z */
@@ -1109,18 +1312,24 @@ long isideal(GEN nf,GEN x)
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
+int fnz(GEN x,long j)
+{
+  long i;
+  i=1;while(!signe((GEN)x[i])) i++;
+  return i==j;
+}
 
 GEN allbase4(GEN f,long code, GEN *y )
 {
-  GEN w,a,da,b,db,bas,q;
-  long v,n,mf,h;
+  GEN w,a,da,b,db,bas,q,bdiag,ab;
+  long v,n,mf,h,templevel;
   long i,j,k,av=avma,tetpil,pro,first;
   
   if(typ(f)!=10) err(allbaser1);
   n=lgef(f)-3;if(n<=0) err(allbaser1);
   v=varn(f);
   *y=discsr(f);
-  
+  if(DEBUGLEVEL) {timer2();templevel=DEBUGLEVEL;DEBUGLEVEL=5;}
   switch(code)
     {
     case 0: w=auxdecomp(absi(*y),1);h=lg((GEN)w[1])-1;break; /* base */
@@ -1131,22 +1340,55 @@ GEN allbase4(GEN f,long code, GEN *y )
       q=gun;for(i=1;i<=h;i++) q=gmul(q,gpui((GEN)coeff(w,i,1),(GEN)coeff(w,i,2)));
       if(gcmp(absi(q),absi((GEN)*y))) err(factoreder2);
     }
-  a=idmat(n);
-  da=gun;
-  first=TRUE;
+  if(DEBUGLEVEL) 
+    {
+      DEBUGLEVEL=templevel;
+      fprintf(errfile,"temps factorisation disc: ");fprintf(errfile,"%ld\n",timer2());fflush(errfile);
+    }
+  a=idmat(n);da=gun;first=TRUE;
   for(i=1;i<=h;i++)
     {
       mf=itos((GEN)coeff(w,i,2));
       if(mf>1)
-	{
+	{ 
+	  if(DEBUGLEVEL)
+	    {
+	      fprintf(errfile,"On traite le cas p^k = ");
+	      temp=outfile;outfile=errfile;
+	      brute((GEN)coeff(w,i,1),'g',-1);
+              fprintf(errfile,"^%ld\n",mf);
+              outfile=temp;
+	    }
 	  b=(GEN)maxord((GEN)coeff(w,i,1),f,mf);
-	  db=denom(b); 
+	  bdiag=cgetg(n+1,17);for(j=1;j<=n;j++) bdiag[j]=coeff(b,j,j);
+	  db=denom(bdiag); 
 	  if (!(gcmp1(db)))    /* la matrice est identite   */
 	    {
 	      da=gmul(da,db);
-	      if (first !=TRUE) a=hnf(concat(gmul(da,b),gmul(a,db))); 
+	      if (first!=TRUE)
+		{
+		  b=gmul(da,b);a=gmul(db,a);
+		  for(j=1;(j<=n)&&(fnz((GEN)a[j],j)&&fnz((GEN)b[j],j));j++);
+		  k=j-1;ab=cgetg(2*n-k+1,19);
+		  for(j=1;j<=k;j++)
+		    {
+		      ab[j]=a[j];
+		      coeff(ab,j,j)=(long)mppgcd(gcoeff(a,j,j),gcoeff(b,j,j));
+		    }
+		  for(;j<=n;j++) ab[j]=a[j];
+		  for(;j<=2*n-k;j++) ab[j]=b[j+k-n];
+		  a=hnf(ab);
+		}
 	      else {a=gmul(b,db);first=FALSE;}
 	    }
+          if(DEBUGLEVEL>=3)
+            {
+              fprintf(errfile,"Le resultat pour ce nombre p est : \n ");
+              temp=outfile;outfile=errfile;
+	      output(b);
+              fprintf(errfile,"\n");
+	      outfile=temp;
+            }
 	}
     } 
   for(j=1;j<=n;j++)
@@ -1169,50 +1411,129 @@ GEN allbase4(GEN f,long code, GEN *y )
 
 GEN maxord(GEN p,GEN f,long mf)
 {
-  GEN w,g,h,res;
-  long j,r,v=varn(f),n=lgef(f)-3,av=avma,tetpil;
+  GEN w,g,h,res,fmp;
+  long j,r,v=varn(f),n=lgef(f)-3,av=avma,tetpil,flw;
 
-  w=factmod(f,p);
-  r=lg((GEN)w[1])-1;
-  g=gun;
-  for(j=1;j<=r;j++)
-    g=gmul((GEN)coeff(w,j,1),g);
-  h=bestnu(w);
-  
-  if (dedek(f,p,g))
+  if(flw=(cmpsi(n,p)<0))
     {
-      avma=av;
-      return idmat(n);
+      fmp=gmul(gmodulcp(gun,p),f);
+      g=gdeuc(fmp,polgcd(fmp,deriv(fmp,v)));
     }
+  else 
+    {
+      w=factmod(f,p);r=lg((GEN)w[1])-1;g=gun;
+      for(j=1;j<=r;j++) g=gmul((GEN)coeff(w,j,1),g);
+    }
+  res=dedek(f,p,g);
+  if(itos((GEN)res[1])) {avma=av;return idmat(n);}
   else
-    {
-      if (lg((GEN)w[1])==2)
-	{ 
-	  tetpil=avma;
-	  res=nilord(p,f,mf,h);
-	  return gerepile(av,tetpil,gcopy(res));
-	}
-      else
-	{ 
-	  tetpil=avma;
-	  return gerepile(av,tetpil,Decomp(p,f,mf,polx[v],f,h));
-	}
-    }
+    if (2*itos((GEN)res[3])+1>=mf)
+      {
+	tetpil=avma;
+	return gerepile(av,tetpil,gdiv((GEN)res[2],p));
+      }
+    else
+      {
+	if(flw) {w=factmod(f,p);r=lg((GEN)w[1])-1;}
+	h=bestnu(w);
+	if (r==1)
+	  { 
+	    res=nilord(p,f,mf,h);
+	    tetpil=avma;return gerepile(av,tetpil,gcopy(res));
+	  }
+	else
+	  { 
+	    tetpil=avma;return gerepile(av,tetpil,Decomp(p,f,mf,polx[v],f,h));
+	  }
+      }
 }
 
-
-long dedek(GEN f,GEN p,GEN g)
+GEN dedek(GEN f,GEN p,GEN g)
 {
-  long av=avma;
-  GEN k,h,unmodp;
+  long av=avma,tetpil,index,dk,n=lgef(f)-3;
+  long dh,c,i,v=varn(f);
+  GEN k,h,unmodp,U,res,b,p1;
   
+  res=cgetg(4,17);
+  if(DEBUGLEVEL>=3)
+    {
+      fprintf(errfile," On est dans Dedekind ");
+      if(DEBUGLEVEL>=4)
+	{
+	  fprintf(errfile," avec les parametres \n" );
+	  temp=outfile;outfile=errfile;
+	  fprintf(errfile," p=");brute(p,'g',-1);
+	  fprintf(errfile,",  f=");brute(f,'g',-1);
+          outfile=temp;
+	}
+      fprintf(errfile,"\n");
+    } 
   unmodp=gmodulcp(gun,p);
-  
+  g=gmul(g,unmodp);
+ 
   h=gdivent(gmul(f,unmodp),g);
   k=gdiv(gsub(lift(f),gmul(lift(g),lift(h))),p);
   k=ggcd(gmul(k,unmodp),ggcd(g,h));
-  avma=av;
-  return (lgef(k)==3)?1:0;
+  dk=lgef(k)-3;
+  if(DEBUGLEVEL>=4)
+   fprintf(errfile," Le pgcd est de degre %ld \n",dk );
+  res[1]= (dk==0)?un:zero;
+  if (dk!=0)
+    {
+      U=gdiv(gmul(f,unmodp),k);
+      h=lift(U);
+      b=cgetg(2*n+1,19);
+      for(c=1;c<=n;c++)
+	{
+	  p1=cgetg(n+1,18);b[c]=(long)p1;
+	  for(i=1;i<=n;i++) p1[i]=(c==i)?(long)p:zero;
+	}
+      for(c=n+1;c<=2*n;c++)
+	{
+	  dh=lgef(h)-3;
+	  p1=cgetg(n+1,18);b[c]=(long)p1;
+	  for(i=1;i<=dh+1;i++) p1[i]=(long)lift(gmul((GEN)h[1+i],unmodp));
+	  for(i=dh+2;i<=n;i++) p1[i]=zero;
+	  if (c!=2*n) {h=gmod(gmul(h,polx[v]),f);} 
+	}
+    if(DEBUGLEVEL>=4)
+	{
+	  fprintf(errfile," On construit un nouvel ordre  \n" );
+	  if(DEBUGLEVEL>=5)
+	    {
+	      temp=outfile;outfile=errfile;
+	      output(b);
+	    }
+	  fprintf(errfile," On fait sa HNF \n");
+	  outfile=temp;
+	}
+/*      b=hnfmod(b,gpuigs(p,n)); */
+      b=hnfmodid(b,p);
+      if(DEBUGLEVEL>=4)
+	{
+          fprintf(errfile," Sa HNF est finie \n");
+	  if(DEBUGLEVEL>=5)
+	    {
+	      temp=outfile;outfile=errfile;
+	      output(b);
+	      outfile=temp;
+	    }
+	}
+
+      res[2]=(long)b;
+      
+      index=0;
+      for(i=1;i<=n;i++)
+	index=index+ggval((GEN)coeff(b,i,i),p);
+      res[3]=lstoi(n-index);
+    }
+  else
+    {
+      res[2]=zero;
+      res[3]=un;
+    }
+  tetpil=avma;
+  return gerepile(av,tetpil,gcopy(res));
 }
 
 GEN Decomp(GEN p,GEN f,long mf,GEN theta,GEN chi,GEN nu)
@@ -1222,10 +1543,26 @@ GEN Decomp(GEN p,GEN f,long mf,GEN theta,GEN chi,GEN nu)
   GEN pk,ph,pmr,pdr,pme,pde;
   GEN b1,b2,b3,a2,a1,e,f1,f2;
   GEN ib1,ib2,ibas,h;
+  GEN df1,df2;
 
+  if(DEBUGLEVEL>=3)
+    {
+      fprintf(errfile," On entre dans Decomp ");
+      if(DEBUGLEVEL>=4)
+	{
+	  temp=outfile;outfile=errfile;
+	  fprintf(errfile," avec les parametres suivants \n ");
+	  fprintf(errfile," p=");brute(p,'g',-1);
+	  fprintf(errfile,",  f=");brute(f,'g',-1);
+	  fprintf(errfile,",  exposant=%ld ",mf);
+	  outfile=temp;
+	}
+      fprintf(errfile,"\n");
+    }
+  
   unmodp=gmodulcp(gun,p);
 
-  pdr=respm(f,deriv(f,v),gpuigs(p,mf));
+  pdr=(GEN)respm(f,deriv(f,v),gpuigs(p,mf));
   pmr=mulii(pdr,mulii(pdr,p));
 
   unmodpmr=gmodulcp(gun,pmr);
@@ -1243,7 +1580,7 @@ GEN Decomp(GEN p,GEN f,long mf,GEN theta,GEN chi,GEN nu)
       b3=lift(gbezout(b2,b1,&a1,&a2));
     }
                            
-  e=eleval(f,lift(gmul(a1,b2)),theta);
+  e=(GEN)eleval(f,lift(gmul(a1,b2)),theta);
   e=gdiv(lift(gmul(gmul(pdr,e),unmodpdrp)),pdr); 
 
   pk=p;
@@ -1259,11 +1596,10 @@ GEN Decomp(GEN p,GEN f,long mf,GEN theta,GEN chi,GEN nu)
       e=gdiv(lift(gmul(gmul(pdr,e),unmodpkpdr)),pdr);
     }  
   
-  f1=gcdpm(f,gmul(pdr,gsubsg(1,e)),mulii(pmr,pdr));f1=lift(gmul(gmod(f1,f),unmodpmr));
-  f2=gdivent(f,f1);                         f2=lift(gmul(gmod(f2,f),unmodpmr));
+  f1=(GEN)gcdpm(f,gmul(pdr,gsubsg(1,e)),mulii(pmr,pdr));f1=lift(gmul(gmod(f1,f),unmodpmr));
+  f2=gdivent(f,f1);f2=lift(gmul(gmod(f2,f),unmodpmr));
 
-
-  n1=lgef(f1)-3;     v1=ggval(discsr(f1),p);    b1=(GEN)maxord(p,f1,v1);
+  n1=lgef(f1)-3;v1=ggval(discsr(f1),p); b1=(GEN)maxord(p,f1,v1);
 
   ib1=cgetg(n1+1,17);
   for(i=1;i<=n1;i++)
@@ -1274,7 +1610,7 @@ GEN Decomp(GEN p,GEN f,long mf,GEN theta,GEN chi,GEN nu)
       ib1[i]=(long)h;
     }  
   
-  n2=lgef(f2)-3;     v2=ggval(discsr(f2),p);    b2=(GEN)maxord(p,f2,v2);
+  n2=lgef(f2)-3; v2=ggval(discsr(f2),p); b2=(GEN)maxord(p,f2,v2);
 
   ib2=cgetg(n2+1,17);
   for(i=1;i<=n2;i++)
@@ -1289,19 +1625,36 @@ GEN Decomp(GEN p,GEN f,long mf,GEN theta,GEN chi,GEN nu)
 
 
   for(j=1;j<=n1;j++)
-    ibas[j]=(long)gmod(lift(gmul(gmul(gmul(pdr,(GEN)ib1[j]),e),unmodpdr)),f);
+    ibas[j]=(long)lift(gmul(gmod(gmul(gmul(pdr,(GEN)ib1[j]),e),f),unmodpdr));
   for(j=n1+1;j<=n1+n2;j++)
-    ibas[j]=(long)gmod(lift(gmul(gmul(gsubsg(1,e),gmul(pdr,(GEN)ib2[j-n1])),unmodpdr)),f);
+    ibas[j]=(long)lift(gmul(gmod(gmul(gsubsg(1,e),gmul(pdr,(GEN)ib2[j-n1])),f),unmodpdr));
   tetpil=avma;
   return gerepile(av,tetpil,nbasis(ibas,pdr));
  
 }
 
+
 GEN nilord(GEN p,GEN fx,long mf,GEN gx)
 {
-  long La,Ma,first,v=varn(fx),av=avma,tetpil; 
-  GEN alpha,chi,nu,eta,w,phi;
-  GEN pm,Dchi,unmodp,unmodpm;
+  long La,Ma,first=TRUE,v=varn(fx),av=avma,tetpil; 
+  GEN alpha,chi,nu,eta,w,phi,U;
+  GEN res,pm,Dchi,unmodp,unmodpm;
+  
+
+  if(DEBUGLEVEL>=3)
+    {
+      fprintf(errfile," On entre dans Nilord ");
+      if(DEBUGLEVEL>=4)
+	{
+	  temp=outfile;outfile=errfile;
+	  fprintf(errfile," avec les parametres suivants \n ");
+	  fprintf(errfile," p=");brute(p,'g',-1);
+	  fprintf(errfile,",  fx=");brute(fx,'g',-1);
+	  fprintf(errfile,",  exposant=%ld,  gx= ",mf);brute(gx,'g',-1);
+          outfile=temp;
+	}
+      fprintf(errfile,"\n");
+    } 
   
   pm=gpuigs(p,mf+1);
 
@@ -1309,53 +1662,62 @@ GEN nilord(GEN p,GEN fx,long mf,GEN gx)
 
   unmodpm=gmodulcp(gun,pm);
   unmodp=gmodulcp(gun,p);
+ 
+  res=cgetg(4,17);
 
   while (TRUE)
     {
       if (gcmp0(Dchi))
 	alpha=gadd(alpha,gmul(p,polx[v]));
       else
-	if (dedek(chi,p,nu)==1) 
-	  { 
-	    tetpil=avma;
-	    return gerepile(av,tetpil,pbasis(p,fx,mf,alpha));
-	  }
-	else
-	  {
-	    if (gcmp(vstar(p,chi),gzero)==1)
-	      {
-		alpha=gadd(alpha,gun);
-		chi=gsubst(chi,v,gsub(polx[v],gun));
-		nu=lift(gmul(gsubst(nu,v,gsub(polx[v],gun)),unmodp));
-	      }
-	    w=setup(p,chi,polx[v],nu);
-	    eta=(GEN)w[2];
-	    La=itos((GEN)w[3]);
-	    Ma=itos((GEN)w[4]);
-	    if (La>1)
-	      alpha=gadd(alpha,eleval(fx,eta,alpha));
-	    else
-	      {
-		w=bsrch(p,chi,ggval(Dchi,p),eta,Ma);
-		phi=eleval(fx,(GEN)w[2],alpha);
-		if (gcmp1((GEN)w[1]))
-		  {
-		    tetpil=avma;
-		    return gerepile(av,tetpil,Decomp(p,fx,mf,phi,(GEN)w[3],(GEN)w[4]));
-		  }
-		else alpha=phi;
-	      }
-	  }
-      w=factcp(p,fx,alpha);
-      chi=(GEN)w[1];nu=(GEN)w[2];
-      if(cmpis((GEN)w[4],1)==1)
 	{
-	  tetpil=avma;
-	  return gerepile(av,tetpil,Decomp(p,fx,mf,alpha,chi,nu));
+         if (first!=TRUE)
+	  {
+	    res=dedek(chi,p,nu);
+	  }
+	 else { res[1]=zero;first=FALSE;}
+	 if (itos((GEN)res[1])==1) 
+	   { 
+	     tetpil=avma;
+	     return gerepile(av,tetpil,pbasis(p,fx,mf,alpha));
+	   }
+	 else
+	   { 
+	     if (gcmp(vstar(p,chi),gzero)==1)
+	       {
+		 alpha=gadd(alpha,gun);
+		  chi=gsubst(chi,v,gsub(polx[v],gun));
+		  nu=lift(gmul(gsubst(nu,v,gsub(polx[v],gun)),unmodp));
+		}
+	      w=(GEN)setup(p,chi,polx[v],nu);
+	      eta=(GEN)w[2];
+	      La=itos((GEN)w[3]);
+	      Ma=itos((GEN)w[4]);
+	      if (La>1)
+		alpha=gadd(alpha,eleval(fx,eta,alpha));
+	      else
+		{
+		  w=(GEN)bsrch(p,chi,ggval(Dchi,p),eta,Ma);
+		  phi=(GEN)eleval(fx,(GEN)w[2],alpha);
+		  if (gcmp1((GEN)w[1]))
+		    {
+		      tetpil=avma;
+		      return gerepile(av,tetpil,Decomp(p,fx,mf,phi,(GEN)w[3],(GEN)w[4]));
+		    }
+		  else alpha=phi;
+		}
+	    }
+	  w=(GEN)factcp(p,fx,alpha);
+	  chi=(GEN)w[1];nu=(GEN)w[2];
+	  if(cmpis((GEN)w[4],1)==1)
+	    {
+	      tetpil=avma;
+	      return gerepile(av,tetpil,Decomp(p,fx,mf,alpha,chi,nu));
+	    }
+	  Dchi=lift(gmul(discsr(lift(gmul(chi,unmodpm))),unmodpm));
+	  if (gcmp0(Dchi))
+	    Dchi=discsr(chi);
 	}
-      Dchi=lift(gmul(discsr(lift(gmul(chi,unmodpm))),unmodpm));
-      if (gcmp0(Dchi))
-	Dchi=discsr(chi);
     }
 }
 
@@ -1378,7 +1740,7 @@ GEN bsrch(GEN p,GEN fa,long ka,GEN eta,long Ma)
   pcc=gmul(pc,pc);
   unmodpcc=gmodulcp(gun,pcc);
   
-  r=1+ceil(c/(double)(Da)+gtodouble(gdivsg(c*n-2,mulsi(Da,subis(p,1)))));
+  r=1+(long)ceil(c/(double)(Da)+gtodouble(gdivsg(c*n-2,mulsi(Da,subis(p,1)))));
   
   b=cgetg(5,17);
   
@@ -1802,7 +2164,7 @@ GEN hnfpm(GEN a,GEN pm)
   
   for(k=1;k<=m;k++)
     {
-      b[k]=lgetg(n+1,17);
+      b[k]=lgetg(n+1,18);
       for(j=1;j<=n;j++)
 	{
 	  coeff(b,j,k)=(long)lift(gmul((GEN)coeff(a,j,k),unmodpm));
@@ -1810,7 +2172,7 @@ GEN hnfpm(GEN a,GEN pm)
     }
   for(k=m+1;k<=m+n;k++)
     {
-      b[k]=lgetg(n+1,17);
+      b[k]=lgetg(n+1,18);
       for(j=1;j<=n;j++)
 	{
 	  coeff(b,j,k)=((j+m)==(k))? (long)pm:zero;
@@ -1821,29 +2183,29 @@ GEN hnfpm(GEN a,GEN pm)
     {
       for(j=k-1;j>=1;j--)
 	if (!(gcmp0((GEN)coeff(b,i,j))))
-	  {	c00=gbezout((GEN)coeff(b,i,k),(GEN)coeff(b,i,j),&c11,&c12);
-		c21=gdiv(gneg((GEN)coeff(b,i,j)),c00);
-		c22=gdiv((GEN)coeff(b,i,k),c00);
-		
-		for(l=1;l<=n;l++)
+	  {
+	    c00=gbezout((GEN)coeff(b,i,k),(GEN)coeff(b,i,j),&c11,&c12);
+	    c21=gdiv(gneg((GEN)coeff(b,i,j)),c00);
+	    c22=gdiv((GEN)coeff(b,i,k),c00);
+	    
+	    for(l=1;l<=n;l++)
+	      { 
+		c13=gadd(gmul(c11,(GEN)coeff(b,l,k)),gmul(c12,(GEN)coeff(b,l,j)));
+		c23=gadd(gmul(c21,(GEN)coeff(b,l,k)),gmul(c22,(GEN)coeff(b,l,j)));
+		if (j<=m)
 		  { 
-		    c13=gadd(gmul(c11,(GEN)coeff(b,l,k)),gmul(c12,(GEN)coeff(b,l,j)));
-		    c23=gadd(gmul(c21,(GEN)coeff(b,l,k)),gmul(c22,(GEN)coeff(b,l,j)));
-		    if (j<=m)
-		      { 
-			coeff(b,l,k)=(long)lift(gmul(c13,unmodpm));
-			coeff(b,l,j)=(long)lift(gmul(c23,unmodpm));
-		      }
-		    else
-		      {
-			coeff(b,l,k)=(long)c13;
-			coeff(b,l,j)=(long)c23;
-		      }
+		    coeff(b,l,k)=(long)lift(gmul(c13,unmodpm));
+		    coeff(b,l,j)=(long)lift(gmul(c23,unmodpm));
+		  }
+		else
+		  {
+		    coeff(b,l,k)=(long)c13;
+		    coeff(b,l,j)=(long)c23;
 		  }
 	      }
+	  }
       if (cmpis((GEN)coeff(b,i,k),0)==-1)
-	for(l=1;l<=n;l++)
-	  coeff(b,l,k)=lneg((GEN)coeff(b,l,k));
+	for(l=1;l<=n;l++) coeff(b,l,k)=lneg((GEN)coeff(b,l,k));
       for(c=k+1;c<=m+n;c++)
 	{
 	  q=gdivent((GEN)coeff(b,i,c),(GEN)coeff(b,i,k));
@@ -1854,8 +2216,7 @@ GEN hnfpm(GEN a,GEN pm)
       k--;
     }
   cb=cgetg(n+1,19);
-  for(k=m+1;k<=m+n;k++)
-    cb[k-m]=b[k];
+  for(k=m+1;k<=m+n;k++) cb[k-m]=b[k];
   tetpil=avma;
   return gerepile(av,tetpil,gcopy(cb));
 }
@@ -1876,7 +2237,7 @@ GEN gcdpm(GEN f1,GEN f2,GEN pm)
   
   h=lift(gmul(gmod(f2,f1),unmodpm));
   for(k=1;k<=n;k++)
-    a[k]=lgetg(n+1,17);
+    a[k]=lgetg(n+1,18);
   for(j=1;j<=n;j++)
     { deg=lgef(h)-3;
       for(k=1;k<=deg+1;k++)
@@ -1941,7 +2302,7 @@ GEN nbasis(GEN ibas,GEN pd)
   for(k=1;k<=n;k++)
     {
       m=lgef((GEN)ibas[k])-2;
-      a[k]=lgetg(n+1,17);
+      a[k]=lgetg(n+1,18);
       for(j=1;j<=m;j++)
 	coeff(a,j,k)=(long)((GEN)ibas[k])[j+1];
       for(j=m+1;j<=n;j++)
@@ -1972,7 +2333,7 @@ GEN pbasis(GEN p,GEN f,long mf,GEN alpha)
   for(k=1;k<=n;k++)
     { 
       dh=lgef(h)-3;
-      a[k]=lgetg(n+1,17);
+      a[k]=lgetg(n+1,18);
       for(j=1;j<=dh+1;j++)
 	coeff(a,j,k)=h[j+1];
       for(j=dh+2;j<=n;j++)
@@ -2028,7 +2389,7 @@ long cbezout(long a,long b,long *u,long *v)
 	  (*v)=(d-a*(*u))/b;
 	  return d;
 	}
-      q=floor(d/(double)v3);
+      q=d/v3;
       r=d%v3;
       t=(*u)-v1*q;
       (*u)=v1;
@@ -2052,7 +2413,7 @@ long clcm(long a,long b)
 	  return (a*b)/d;
 	}
       v1=r;
-      q=floor(d/(double)r);
+      q=d/r;
       r=d%r;
       d=labs(v1);
     }
@@ -2076,11 +2437,15 @@ GEN idealfactor(GEN nf, GEN x)
     {
       p1=primedec(nf,(GEN)f1[i]);ls=itos((GEN)f2[i]);
       vd=ggval(denx,(GEN)f1[i]);
-      for(j=1;(j<lg(p1))&&ls;j++)
+      for(j=1;j<lg(p1);j++)
 	{
-	  v=idealval(nf,x,p2=(GEN)p1[j]);
-	  ls-=(v*itos((GEN)p2[4]));
-	  v-=vd*itos((GEN)p2[3]);
+	  if(ls)
+	    {
+	      v=idealval(nf,x,p2=(GEN)p1[j]);
+	      ls-=(v*itos((GEN)p2[4]));
+	      v-=vd*itos((GEN)p2[3]);
+	    }
+	  else v=-vd*itos((GEN)p2[3]);
 	  if(v) {y1[++k]=(long)p2;y2[k]=v;}
 	}
     }
@@ -2159,6 +2524,7 @@ GEN principalideal(GEN nf, GEN a)
       tetpil=avma;y=gmul((GEN)nf[8],z);z=cgetg(2,19);z[1]=(long)y;
       return gerepile(av,tetpil,z);
     default: err(talker,"incorrect type in principalideal");
+      return gnil;
     }
 }
 
@@ -2188,11 +2554,6 @@ GEN principalidele(GEN nf, GEN a)
       res=cgetg(3,17);res[1]=(long)z;res[2]=lcopy(arc);
       return gerepile(av,tetpil,res);
     default: err(talker,"incorrect type in principalidele");
+      return gnil;
     }
 }
-
-
-  
-
-
-
