@@ -852,7 +852,26 @@ GEN lllgramall(GEN x, long all)
   GEN u,B,lam,q,r,h,la,bb,p2,p3,p4,y,fl;
 
   if(tx!=19) err(lllger1);
-  n=lx-1;if(n<=1) return idmat(n);
+  n=lx-1;
+  if(!n)
+    {
+      if(all) return cgetg(1,19);
+      else {y=cgetg(3,17);y[1]=lgetg(1,19);y[2]=lgetg(1,19);return y;}
+    }
+  if(n==1)
+    {
+      if(gcmp0(gcoeff(x,1,1))) 
+	{
+	  if(!all)
+	    {y=cgetg(3,17);y[1]=(long)idmat(1);y[2]=lgetg(1,19);return y;}
+	  return (all==1)?idmat(1):cgetg(1,19);
+	}
+      else
+	{
+	  if(!all) {y=cgetg(3,17);y[1]=lgetg(1,19);y[2]=(long)idmat(1);return y;}
+	  return (all==1)?cgetg(1,19):idmat(1);
+	}
+    }
   if(lg((GEN)x[1])!=lx) err(lllger2);
   av=avma;lim=(avma+bot)>>1;
   B=cgetg(lx+1,18);
