@@ -20,32 +20,32 @@ GEN rectdraw(GEN list)
   return gnil;
 }
 
-GEN buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO)
+GEN buchimag(GEN D, GEN gcbach, GEN gcbach2)
 {
   err(bucher13);return gnil;
 }
 
-GEN buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gCO, long prec)
+GEN buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, long prec)
 {
   err(bucher13);return gnil;
 }
 
-GEN buchgen(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
+GEN buchgen(GEN P, GEN gcbach, GEN gcbach2, long prec)
 {
   err(bucher13);return gnil;
 }
 
-GEN buchgenfu(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
+GEN buchgenfu(GEN P, GEN gcbach, GEN gcbach2, long prec)
 {
   err(bucher13);return gnil;
 }
 
-GEN buchinit(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
+GEN buchinit(GEN P, GEN gcbach, GEN gcbach2, long prec)
 {
   err(bucher13);return gnil;
 }
 
-GEN buchinitfu(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec)
+GEN buchinitfu(GEN P, GEN gcbach, GEN gcbach2, long prec)
 {
   err(bucher13);return gnil;
 }
@@ -59,3 +59,8 @@ GEN isunit(GEN bnf, GEN x)
 {
   err(bucher13);return gnil;
 }
+
+GEN signunit(GEN bignf)
+{
+  err(bucher13);return gnil;
+}
