@@ -514,7 +514,7 @@ GEN bilhell(GEN e, GEN z1, GEN z2, long prec)
       h2=ghell(e,z2,prec);
       tetpil=avma;return gerepile(av,tetpil,bilhells(e,z1,z2,h2,prec));
     }
-  else err(talker,"two vector/matrix types in bilhell");
+  else {err(talker,"two vector/matrix types in bilhell");return gnil;}
 }
 
 GEN zell(GEN e, GEN z, long prec)
@@ -605,7 +605,7 @@ GEN zell(GEN e, GEN z, long prec)
 
 GEN pointell(GEN e, GEN z, long prec)
 {
-  long av=avma,tetpil;
+  long av=avma,tetpil,dec,lim,av1;
   GEN p1,pii2,pii4,pii6,a,tau,q,u,y,yp,u1,u2,qn,qnu2,qnu1,qnu,qnu4,qnu3,p2,v;
 
   if((typ(e)!=17)||(lg(e)<20)) err(elliper1);
@@ -618,6 +618,7 @@ GEN pointell(GEN e, GEN z, long prec)
   q=gexp(gmul(pii2,tau),prec);u=gexp(gmul(pii2,z),prec);
   y=gadd(gdivgs(gun,12),gdiv(u,u2=gsqr(u1=gsubsg(1,u))));
   yp=gdiv(gaddsg(1,u),gmul(u1,u2));
+  lim=(avma+bot)>>1;av1=avma;
   qn=q;pii2=gdiv(pii2,(GEN)e[15]);pii4=gmul(pii2,pii2);pii6=gmul(pii4,pii2);
   do
     {
@@ -625,6 +626,12 @@ GEN pointell(GEN e, GEN z, long prec)
       y=gadd(y,gmul(qn,p1));
       p2=gadd(gdiv(gaddsg(1,qnu),gmul(qnu1,qnu2)),gdiv(gadd(qn,u),gmul(qnu3,qnu4)));
       yp=gadd(yp,gmul(qn,p2));qn=gmul(q,qn);
+      if(avma<lim)
+	{
+	  tetpil=avma;y=gcopy(y);yp=gcopy(yp);qn=gcopy(qn);
+	  dec=lpile(av1,tetpil,0)>>TWOPOTBYTES_IN_LONG;
+	  y+=dec;yp+=dec;qn+=dec;
+	}
     }
   while(gexpo(qn)>-((prec-2)<<TWOPOTBITS_IN_LONG)-5);
   yp=gmul(u,gmul(pii6,yp));y=gsub(gmul(pii4,y),gdivgs((GEN)e[6],12));
