@@ -170,7 +170,7 @@ char *errmessage[]=
   "bug2 in galois",
   "bug3 in galois",
   "bug4 in galois",
-  "you have found a counterexample to a conjecture,\nplease send us the polynomial as soon as possible",
+  "you have found a counterexample to a conjecture, please send us\nthe polynomial as soon as possible",
   "bug1 in initalg",
   "bug: different degrees in gpolcomp",
   "bug in rootsof1, please report",
@@ -179,7 +179,7 @@ char *errmessage[]=
   "not an ideal in an ideal-related function",
   "not a prime ideal in idealmulprime",
   "not a square matrix in idealmulprime",
-  "not a vector in ideallllred or minideal",
+  "not a vector or incorrect vector length in ideallllred or minideal",
   "polynomial not in Z[X] in initalg0",
   "not a monic polynomial in initalg0",
 
@@ -429,7 +429,7 @@ char *errmessage[]=
   "user interrupt",
   "no more blocks available",
   "function already exists in install",
-  "no memory available for stack doubling",
+  "no memory available for stack expanding",
 
   /*              ERREURS  DE  PLOT.C          */
 
