@@ -300,10 +300,11 @@ GEN   gadd(GEN x, GEN y)
 		    r=r1;z[3]=lcopy((GEN)x[3]);
 		  }
 		else z[3]=lmul(p1,(GEN)y[3]);
-		setprecp(z,r);z[4]=lgeti(lgef((GEN)z[3]));
+		z[4]=lgeti(lgef((GEN)z[3]));
 		l2=avma;p1=mulii(p1,(GEN)y[4]);
 		modiiz(addii(p1,(GEN)x[4]),(GEN)z[3],(GEN)z[4]);avma=l2;
-		setvalp(z,e); z=gerepile(l,tetpil,z);
+		z[1]=evalprecp(r)+evalvalp(e);
+		z=gerepile(l,tetpil,z);
 	      }
 	    else
 	      {
@@ -312,8 +313,7 @@ GEN   gadd(GEN x, GEN y)
 		r=r1;if(r2<r1) {r=r2;p2=x;x=y;y=p2;}
 		if(gcmp0(p1)||((co=pvaluation(p1,p,&p2))>=r))
 		  {
-		    avma=l;setprecp(z,0);z[3]=un;
-		    setvalp(z,e+r);
+		    avma=l;z[1]=evalvalp(e+r);z[3]=un;
 		    z[4]=lgeti(lgef((GEN)x[3]));
 		    affsi(0,(GEN)z[4]);
 		  }
@@ -323,7 +323,7 @@ GEN   gadd(GEN x, GEN y)
 		      {
 			p1=gpuigs(p,co);tetpil=avma;
 			z[3]=ldivii((GEN)x[3],p1);
-			r-=co;setprecp(z,r);setvalp(z,e+co);
+			r-=co;z[1]=evalprecp(r)+evalvalp(e+co);
 			l2=lgef((GEN)z[3]);z[4]=lgeti(l2);
 			modiiz(p2,(GEN)z[3],(GEN)z[4]);
 			gerepile(l,tetpil,(GEN)1);
@@ -334,7 +334,7 @@ GEN   gadd(GEN x, GEN y)
 			modiiz(p1,(GEN)x[3],(GEN)z[4]);
 			z[4]=lpile(l,tetpil,(GEN)z[4]);
 			z[3]=lcopy((GEN)x[3]);
-			setprecp(z,r);setvalp(z,e);
+			z[1]=evalprecp(r)+evalvalp(e);
 		      }
 		  }
 	      }
@@ -395,20 +395,25 @@ GEN   gadd(GEN x, GEN y)
 	    case 10: e=lgef(y);
 	      if(e==2)
 		{
-		  if(isexactzero(x)) {z=cgetg(2,ty);z[1]=evallgef(2);}
+		  if(isexactzero(x)) 
+		    {
+		      z=cgetg(2,ty);z[1]=evallgef(2)+evalvarn(vy);
+		    }
 		  else
 		    {
 		      z=cgetg(3,ty);z[2]=lcopy(x);
-		      z[1]=(gcmp0(x))?evallgef(3):evalsigne(1)+evallgef(3);
+		      z[1]=(gcmp0(x))?evallgef(3)+evalvarn(vy):evalsigne(1)+evallgef(3)+evalvarn(vy);
 		    }
-		  setvarn(z,vy);
 		}
 	      else
 		{
 		  z=cgetg(e,ty);
 		  z[2]=ladd(x,(GEN)y[2]);
 		  if((e==3)&&(gcmp0((GEN)z[2])))
-		    {z[1]=isexactzero((GEN)z[2])?2:3;setvarn(z,vy);}
+		    {
+		      z[1]=isexactzero((GEN)z[2])?evallgef(2)+evalvarn(vy):
+			evallgef(3)+evalvarn(vy);
+		    }
 		  else
 		    {
 		      z[1]=y[1];for(i=3;i<e;i++) z[i]=lcopy((GEN)y[i]);
@@ -440,8 +445,8 @@ GEN   gadd(GEN x, GEN y)
 			    {
 			      if(gcmp0(y)) lz=e+2;else lz=ly+e;
 			      z=cgetg(lz,ty);
-			      z[1]=evalsigne(1)+HIGHVALPBIT;z[2]=lcopy(x);
-			      setvarn(z,vy);
+			      z[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(vy);
+			      z[2]=lcopy(x);
 			      for (i=3;i<=e+1;i++) z[i]=zero;
 			      for (i=e+2;i<lz;i++)
 				z[i]=lcopy((GEN)y[i-e]);
@@ -455,7 +460,7 @@ GEN   gadd(GEN x, GEN y)
 			    {
 			      avma=l;
 			      if(ly==3)
-				{z[2]=zero;z[1]=HIGHVALPBIT+1;}
+				{z[2]=zero;z[1]=HIGHVALPBIT+1+evalvarn(vy);}
 			      else
 				{
 				  i=3;
@@ -463,12 +468,12 @@ GEN   gadd(GEN x, GEN y)
 				  if (i==ly)
 				    {
 				      cgiv(z);z=cgetg(3,ty);z[2]=zero;
-				      z[1]=HIGHVALPBIT-2+i;
+				      z[1]=HIGHVALPBIT-2+i+evalvarn(vy);
 				    }
 				  else
 				    {
 				      cgiv(z);z=cgetg(ly-i+2,ty);
-				      setvalp(z,i-2);setsigne(z,1);
+				      z[1]=evalvalp(i-2)+evalsigne(1)+evalvarn(vy);
 				      for (j=2;j<=ly-i+1;j++)
 					z[j]=lcopy((GEN)y[j+i-2]);
 				    }
@@ -476,15 +481,15 @@ GEN   gadd(GEN x, GEN y)
 			    }
 			  else
 			    {
-			      if(!signe(y)) z[1]=HIGHVALPBIT;
+			      if(!signe(y)) z[1]=HIGHVALPBIT+evalvarn(vy);
 			      else
 				{
-				  z[1]=evalsigne(1)+HIGHVALPBIT;z[2]=(long)p1;
+				  z[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(vy);
+				  z[2]=(long)p1;
 				  for (i=3;i<ly;i++)
 				    z[i]=lcopy((GEN)y[i]);
 				}
 			    }
-			  setvarn(z,vy);
 			}
 		    }
 		}
@@ -582,7 +587,7 @@ GEN   gadd(GEN x, GEN y)
 		    if(e)
 		      {
 			z=cgetg(lz,ty);
-			setvalp(z,valp(x));setsigne(z,1);
+			z[1]=evalvalp(valp(x))+evalsigne(1)+evalvarn(vx);
 			if(e<lz-2)
 			  {
 			    for (i=2;i<=e+1;i++) z[i]=lcopy((GEN)x[i]);
@@ -598,22 +603,21 @@ GEN   gadd(GEN x, GEN y)
 			  {
 			    avma=l; p1=gadd((GEN)x[i],(GEN)y[i]);
 			    f=isexactzero(p1);i++;
-			    /*			    f=gcmp0(p1);i++; */
 			  }
 			if(f)
 			  {
-			    avma=l;z=cgetg(lz,ty);z[1]=HIGHVALPBIT-2+lz+valp(y);
+			    avma=l;z=cgetg(lz,ty);
+			    z[1]=evalvalp(lz-2+valp(y))+evalvarn(vx);
 			  }
 			else
 			  {
 			    z=cgetg(lz-i+3,ty);
-			    setvalp(z,valp(x)+i-3);
-			    setsigne(z,1);z[2]=(long)p1;
+			    z[1]=evalvalp(valp(x)+i-3)+evalsigne(1)+evalvarn(vx);
+			    z[2]=(long)p1;
 			    for(j=i;j<lz;j++)
 			      z[j-i+3]=ladd((GEN)x[j],(GEN)y[j]);
 			  }
 		      }
-		    setvarn(z,vx);
 		  }
 		break;
 		
@@ -1049,7 +1053,7 @@ GEN   gmul(GEN x, GEN y)
 	    {
 	    case 10: z=cgetg(ly,ty);
 	      if (isexactzero(x)||isexactzero(y))
-		{z[1]=evallgef(2);setvarn(z,vy);}
+		{z[1]=evallgef(2)+evalvarn(vy);}
 	      else
 		{
 		  for (i=2;i<lgef(y);i++)
@@ -1061,7 +1065,7 @@ GEN   gmul(GEN x, GEN y)
 	      
 	    case 11: if (gcmp0(x))
 	      {
-		z=cgetg(2,10);z[1]=evallgef(2);setvarn(z,vy);
+		z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(vy);
 	      }
 	    else
 	      {
@@ -1078,7 +1082,7 @@ GEN   gmul(GEN x, GEN y)
 	      break;
 	      
 	    case 13:
-	      if(isexactzero(x)) {z=cgetg(2,10);z[1]=evallgef(2);setvarn(z,vy);}
+	      if(isexactzero(x)) {z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(vy);}
 	      else 
 		{
 		  l=avma;z=cgetg(ly,ty);z[1]=lmul(x,(GEN)y[1]);z[2]=y[2];
@@ -1086,7 +1090,7 @@ GEN   gmul(GEN x, GEN y)
 		}
 	      break;
 	    case 14: 
-	      if(isexactzero(x)) {z=cgetg(2,10);z[1]=evallgef(2);setvarn(z,vy);}
+	      if(isexactzero(x)) {z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(vy);}
 	      else {z=cgetg(ly,ty);z[1]=lmul(x,(GEN)y[1]);z[2]=lcopy((GEN)y[2]);}
 	      break;
 	      
@@ -1113,7 +1117,7 @@ GEN   gmul(GEN x, GEN y)
 	      {
 	      case 10: if (isexactzero(x)||isexactzero(y))
 		{
-		  z=cgetg(2,ty);z[1]=evallgef(2);
+		  z=cgetg(2,ty);z[1]=evallgef(2)+evalvarn(vx);
 		}
 	      else
 		{
@@ -1157,21 +1161,21 @@ GEN   gmul(GEN x, GEN y)
 			}
 		      z[i]=lpile(l,tetpil,p1);
 		    }
-		  z[1]=evalsigne(1)+evallgef(k);
+		  z[1]=evalsigne(1)+evallgef(k)+evalvarn(vx);
 		}
-		setvarn(z,vx);normalizepol(&z);
+		normalizepol(&z);
 		break;
 		
 	      case 11: if (gcmp0(x))
 		{
-		  z=cgetg(2,10);z[1]=evallgef(2);setvarn(z,vx);
+		  z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(vx);
 		}
 	      else
 		{
 		  if (gcmp0(y))
 		    {
-		      z=cgetg(3,11);z[1]=HIGHVALPBIT+valp(y)+gval(x,vx);
-		      setvarn(z,vx);
+		      z=cgetg(3,11);
+		      z[1]=evalvalp(valp(y)+gval(x,vx))+evalvarn(vx);
 		    }
 		  else
 		    {
@@ -1206,14 +1210,12 @@ GEN   gmul(GEN x, GEN y)
 	      case 11: if(lx>ly) {k=ly;ly=lx;lx=k;p1=y;y=x;x=p1;}
 		if (gcmp0(x)||gcmp0(y))
 		  {
-		    z=cgetg(3,11);
-		    z[1]=HIGHVALPBIT+valp(x)+valp(y);
-		    setvarn(z,vx);
+		    z=cgetg(3,11);z[1]=evalvalp(valp(x)+valp(y))+evalvarn(vx);
 		  }
 		else
 		  {
 		    l=avma;p1=cgeti(lx);z=cgetg(lx,ty);
-		    setvalp(z,valp(x)+valp(y));setvarn(z,vx);setsigne(z,1);
+		    z[1]=evalvalp(valp(x)+valp(y))+evalvarn(vx)+evalsigne(1);
 		    for (i=2;i<lx;i++)
 		      {
 			p1[i]=!isexactzero((GEN)y[i]);
@@ -1231,14 +1233,14 @@ GEN   gmul(GEN x, GEN y)
 	      case 13:
 	      case 14: if (gcmp0(y))
 		{
-		  z=cgetg(2,10);z[1]=evallgef(2);setvarn(z,vx);
+		  z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(vx);
 		}
 	      else
 		{
 		  if (gcmp0(x))
 		    {
-		      z=cgetg(3,11);z[1]=HIGHVALPBIT+valp(x)+gval(y,vx);
-		      setvarn(z,vx);
+		      z=cgetg(3,11);
+		      z[1]=evalvalp(valp(x)+gval(y,vx))+evalvarn(vx);
 		    }
 		  else
 		    {
@@ -1828,7 +1830,7 @@ GEN   gdiv(GEN x, GEN y)
 		    if(lgef(y)>3)
 		      {
 			if(isexactzero(x)) 
-			  {z=cgetg(2,10);z[1]=evallgef(2);setvarn(z,vy);}
+			  {z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(vy);}
 			else
 			  {
 			    l=avma;z=cgetg(3,13);z[1]=(long)x;
@@ -1847,7 +1849,7 @@ GEN   gdiv(GEN x, GEN y)
 		    else
 		      {
 			p1=cgetg(ly,ty);p1[2]=lcopy(x);
-			p1[1]=evalsigne(1)+HIGHVALPBIT;setvarn(p1,vy);
+			p1[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(vy);
 			for(i=3;i<ly;i++) p1[i]=zero;
 			tetpil=avma;p2=gdiv(p1,y);
 			z=gerepile(l,tetpil,p2);
@@ -1877,7 +1879,7 @@ GEN   gdiv(GEN x, GEN y)
 		      break;
 		    case 11: if(gcmp0(x))
 		      {
-			z=cgetg(2,10);z[1]=evallgef(2);setvarn(z,vx);
+			z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(vx);
 		      }
 		    else
 		      {
@@ -1904,14 +1906,14 @@ GEN   gdiv(GEN x, GEN y)
 		    case 11: lz=lx;if(ly<lx) lz=ly;
 		      if(gcmp0(x))
 			{
-			  z=cgetg(3,11);z[1]=HIGHVALPBIT+valp(x)-valp(y);
-			  setvarn(z,vx);
+			  z=cgetg(3,11);
+			  z[1]=evalvalp(valp(x)-valp(y))+evalvarn(vx);
 			}
 		      else
 			{
 			  z=cgetg(lz,ty);p3=cgeti(lz);
 			  for(i=3;i<lz;i++) p3[i]=!isexactzero((GEN)y[i]);
-			  setvalp(z,valp(x)-valp(y));setvarn(z,vx);
+			  z[1]=evalvalp(valp(x)-valp(y))+evalvarn(vx);
 			  z[2]=ldiv((GEN)x[2],(GEN)y[2]);
 			  for(i=3;i<lz;i++)
 			    {
