@@ -249,10 +249,9 @@ GEN cgetp(GEN x)
 {
   GEN y;
   
-  y=cgetg(5,7);
-  y[2] = (long)copyifstack((GEN)x[2]); y[3] = lcopy((GEN)x[3]);
-  setprecp(y, precp(x)); y[4] = lgeti(lg((GEN)x[3]));
-  return y;
+  y=cgetg(5,7);y[1]=evalprecp(precp(x));
+  y[2]=(long)copyifstack((GEN)x[2]);y[3]=lcopy((GEN)x[3]);
+  y[4]=lgeti(lg((GEN)x[3]));return y;
 }
 
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
@@ -363,7 +362,7 @@ GEN gclone(GEN x)
 
 GEN     greffe(GEN x, long l)
 {
-  long    i,e,k;
+  long    i,e,k,vx;
   GEN     y;
   
   if (typ(x)!=10) err(grefer1);
@@ -372,12 +371,12 @@ GEN     greffe(GEN x, long l)
     y=cgetg(l,11);
     if (gcmp0(x))
     {
-      y[1]=HIGHVALPBIT-2+l;
+      y[1]=evalvalp(l-2)+evalvarn(varn(x));
       for (i=2;i<l;y[i]=x[2],i++);
     }
     else
     {
-      e=gval(x,varn(x));y[1]=evalsigne(1)+HIGHVALPBIT+e;
+      e=gval(x,vx=varn(x));y[1]=evalsigne(1)+evalvalp(e)+evalvarn(vx);
       k=lgef(x)-e-1;
       if (k<l)
       {
@@ -388,7 +387,7 @@ GEN     greffe(GEN x, long l)
         for (i=2;i<l;y[i]=x[i+e],i++);
     }
   }
-  setvarn(y,varn(x));return y;
+  return y;
 }
 
 
@@ -805,20 +804,20 @@ long pvaluation(GEN x, GEN p, GEN *py)
 
 GEN     gneg(GEN x)
 {
-  long  tx,lx,i,sx;
+  long  tx=typ(x),lx,i,sx;
   GEN   y;
   
   if (gcmp0(x))
   {
-    y=gcopy(x);
+    y=(tx==1)?gzero:gcopy(x);
   }
   else
   {
-    tx=typ(x);lx=lg(x);
+    lx=lg(x);
     
     switch(tx)
     {
-      case 1 :
+      case 1 : 
       case 2 : y=gcopy(x);
         sx=signe(x);
         sx= -sx;
@@ -1046,21 +1045,21 @@ void gaffsg(long s, GEN x)
     case 9 : gaffsg(s,(GEN)x[2]);break;
       
     case 10: v=varn(x);
-      if (!s) x[1]=evallgef(2);
+      if (!s) x[1]=evallgef(2)+evalvarn(v);
       else
       {
-        x[1]=evalsigne(1)+evallgef(3);gaffsg(s,(GEN)x[2]);
+        x[1]=evalsigne(1)+evallgef(3)+evalvarn(v);gaffsg(s,(GEN)x[2]);
       }
-      setvarn(x,v);break;
+      break;
       
     case 11: v=varn(x);gaffsg(s,(GEN)x[2]);l=lg(x);
-      if (!s) x[1]=HIGHVALPBIT-2+l;
+      if (!s) x[1]=evalvalp(l-2)+evalvarn(v);
       else
       {
-        x[1]=evalsigne(1)+HIGHVALPBIT;
+        x[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(v);
         for (i=3;i<l;gaffsg(0,(GEN)x[i]),i++);
       }
-      setvarn(x,v);break;
+      break;
       
     case 13:
     case 14: gaffsg(s,(GEN)x[1]);gaffsg(1,(GEN)x[2]);
@@ -1104,20 +1103,23 @@ void gaffect(GEN x, GEN y)
     {
       switch(ty)
       {
-        case 10: v=varn(y);gaffect(x,(GEN)y[2]);
+        case 10: v=varn(y);
+	  if((y==polun[v])||(y==polx[v])) 
+	    err(talker,"trying to overwrite universal polynomial, please report!");
+	  gaffect(x,(GEN)y[2]);
           for(i=3;i<ly;gaffsg(0,(GEN)y[i]),i++);
-          if (gcmp0(x)) y[1]=evallgef(2);
-          else y[1]=evalsigne(1)+evallgef(3);
-          setvarn(y,v);break;
+          if (gcmp0(x)) y[1]=evallgef(2)+evalvarn(v);
+          else y[1]=evalsigne(1)+evallgef(3)+evalvarn(v);
+          break;
           
         case 11: v=varn(y);gaffect(x,(GEN)y[2]);
-          if (gcmp0(x)) y[1]=HIGHVALPBIT-2+ly;
+          if (gcmp0(x)) y[1]=evalvalp(ly-2)+evalvarn(v);
           else
           {
-            y[1]=evalsigne(1)+HIGHVALPBIT;
+            y[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(v);
             for (i=3;i<ly;gaffsg(0,(GEN)y[i]),i++);
           }
-          setvarn(y,v);break;
+          break;
           
         case 13:
         case 14: gaffect(x,(GEN)y[1]);gaffsg(1,(GEN)y[2]); break;
@@ -1135,12 +1137,20 @@ void gaffect(GEN x, GEN y)
         case 1 :
           switch(ty)
           {
-            case 1 : affii(x,y);break;
-            case 2 : affir(x,y);break;
+            case 1 : if(((y==gnil)||(y==gzero)||(y==gun)||(y==gdeux))&&(x!=y))
+	      err(talker,"trying to overwrite a universal integer, please report!");
+	      affii(x,y);break;
+            case 2 : if((y==gpi)||(y==geuler))
+	      err(talker,"trying to overwrite a universal real, please report!");
+	      affir(x,y);break;
             case 3 : modiiz(x,(GEN)y[1],(GEN)y[2]);break;
             case 4 :
-            case 5 : affii(x,(GEN)y[1]);affsi(1,(GEN)y[2]); break;
-            case 6 : gaffect(x,(GEN)y[1]);gaffsg(0,(GEN)y[2]); break;
+            case 5 : if(y==ghalf)
+	      err(talker,"trying to overwrite a universal fraction, please report!");
+	      affii(x,(GEN)y[1]);affsi(1,(GEN)y[2]); break;
+            case 6 : if(y==gi)
+	      err(talker,"trying to overwrite the universal i, please report!");
+	      gaffect(x,(GEN)y[1]);gaffsg(0,(GEN)y[2]); break;
             case 7 : if (!signe(x)) {setvalp(y,defaultpadicprecision);y[4]=zero;}
               else
               {
@@ -1362,9 +1372,8 @@ void gaffect(GEN x, GEN y)
             {
               gaffect(x,(GEN)y[2]);
               for(i=3;i<ly;gaffsg(0,(GEN)y[i]),i++);
-              if (signe(x)) y[1]=evalsigne(1)+evallgef(3);
-              else y[1]=evallgef(2);
-              setvarn(y,vy);
+              if (signe(x)) y[1]=evalsigne(1)+evallgef(3)+evalvarn(vy);
+              else y[1]=evallgef(2)+evalvarn(vy);
             }
             break;
             
@@ -1374,7 +1383,7 @@ void gaffect(GEN x, GEN y)
             {
               if(vy==v)
               {
-                i=gval(x,v);setvalp(y,i);setsigne(y,1);
+                i=gval(x,v);y[1]=evalvarn(v)+evalvalp(i)+evalsigne(1);
                 k=lgef(x)-i;
                 if(k>ly) k=ly;
                 for (j=2;j<k;j++)
@@ -1385,13 +1394,12 @@ void gaffect(GEN x, GEN y)
               else
               {
                 gaffect(x,(GEN)y[2]);
-                if (!signe(x)) y[1]=HIGHVALPBIT-2+ly;
+                if (!signe(x)) y[1]=evalvalp(ly-2)+evalvarn(vy);
                 else
                 {
-                  y[1]=evalsigne(1)+HIGHVALPBIT;
+                  y[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(vy);
                   for (i=3;i<ly;gaffsg(0,(GEN)y[i]),i++);
                 }
-                setvarn(y,vy);
               }
             }
             break;
@@ -1422,13 +1430,12 @@ void gaffect(GEN x, GEN y)
         else
         {
           gaffect(x,(GEN)y[2]);
-          if (!signe(x)) y[1]=HIGHVALPBIT-2+ly;
+          if (!signe(x)) y[1]=evalvalp(ly-2)+evalvarn(vy);
           else
           {
-            y[1]=evalsigne(1)+HIGHVALPBIT;
+            y[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(vy);
             for (i=3;i<ly;gaffsg(0,(GEN)y[i]),i++);
           }
-          setvarn(y,vy);
         }
         break;
         
@@ -1631,19 +1638,18 @@ void normalize(GEN *px)
     if (f)
     {
       avma=l;
-      p1=cgetg(3,11);p1[1]=HIGHVALPBIT-2+lx;
+      p1=cgetg(3,11);p1[1]=evalvalp(lx-2)+evalvarn(v);
       *px=p1;
     }
     else
     {
       tetpil=avma;
       p1=cgetg(lx-i+2,11);
-      p1[1]=evalsigne(1)+HIGHVALPBIT-2+e+i;
+      p1[1]=evalsigne(1)+evalvalp(e+i-2)+evalvarn(v);
       for (j=i;j<lx;j++)
         p1[j-i+2]=lcopy((GEN)x[j]);
       *px=gerepile(l,tetpil,p1);
     }
-    setvarn(*px,v);
   }
   else
   {
@@ -1796,8 +1802,9 @@ GEN     gpui(GEN x, GEN n, ...)
 		if(gcmp1((GEN)x[2]))
 		  {
 		    av1=avma;alp=gaddgs(n,1);
-		    av2=avma;y=cgetg(lx=lg(x),11);y[1]=evalsigne(1)+HIGHVALPBIT;
-		    y[2]=un;setvarn(y,varn(x));
+		    av2=avma;y=cgetg(lx=lg(x),11);
+		    y[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(varn(x));
+		    y[2]=un;
 		    for(i=3;i<lx;i++)
 		      {
 			av3=avma;p1=gzero;
@@ -1899,9 +1906,10 @@ GEN     gpuigs(GEN x, long n, ...)
     {
       j=lgef(x);
       dd=(j-3)*m+3;
-      av=avma;p1=cgetg(dd,10);p1[1]=evalsigne(1)+evallgef(dd);setvarn(p1,varn(x));
+      av=avma;
+      p1=cgetg(dd,10);p1[1]=evalsigne(1)+evallgef(dd)+evalvarn(varn(x));
+      for(i=2;i<=dd-1;i++) p1[i]=zero;
       p1[dd-1]=lpuigs((GEN)x[j-1],m);
-      for(i=2;i<dd-1;i++) p1[i]=zero;
       if(n<0)
       {
         tetpil=avma;y=cgetg(3,13);y[1]=(long)denom((GEN)p1[dd-1]);
@@ -1995,17 +2003,15 @@ GEN     gsqr(GEN x)
         j=vali(p1);if(j<i) i=j;
       }
       avma=av;y[3]=lshifti((GEN)x[3],i);
-      setprecp(y,precp(x)+i);
+      y[1]=evalprecp(precp(x)+i)+evalvalp(2*valp(x));
     }
     else
     {
       y[3]=lcopy((GEN)x[3]);
-      setprecp(y,precp(x));
+      y[1]=evalprecp(precp(x))+evalvalp(2*valp(x));
     }
     y[4]=lgeti(lg((GEN)y[3]));
-    setvalp(y,2*valp(x));av=avma;
-    modiiz(mulii((GEN)x[4],(GEN)x[4]),(GEN)y[3],(GEN)y[4]);
-    avma=av;
+    av=avma;modiiz(mulii((GEN)x[4],(GEN)x[4]),(GEN)y[3],(GEN)y[4]);avma=av;
   }
   else if(tx==15) y=sqcompreal(x);
   else if(tx==16) y=sqcomp(x);
