@@ -201,7 +201,7 @@ GEN   gmulsg(long s, GEN y)
     break;
   
   case 10: 
-    if ((!s)||gcmp0(y)) {z=cgetg(2,10);z[1]=2;setvarn(z,varn(y));}
+    if ((!s)||gcmp0(y)) {z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(varn(y));}
     else
     {
       ly=lgef(y);z=cgetg(ly,ty);
@@ -213,7 +213,7 @@ GEN   gmulsg(long s, GEN y)
   
   case 11: if (!s)
     {
-    z=cgetg(3,10);z[1]=2;setvarn(z,varn(y));
+    z=cgetg(3,10);z[1]=evallgef(2)+evalvarn(varn(y));
     }
   else
     {
@@ -235,11 +235,11 @@ GEN   gmulsg(long s, GEN y)
 	l=avma;z=cgetg(ly,ty);z[1]=lmulsg(s,(GEN)y[1]);z[2]=y[2];
 	tetpil=avma;z=gerepile(l,tetpil,gred(z));
       }
-    else {z=cgetg(2,10);z[1]=2;setvarn(z,gvar(y));}
+    else {z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(gvar(y));}
     break;
   case 14: 
     if(s) {z=cgetg(ly,ty);z[1]=lmulsg(s,(GEN)y[1]);z[2]=lcopy((GEN)y[2]);}
-    else {z=cgetg(2,10);z[1]=2;setvarn(z,gvar(y));}
+    else {z=cgetg(2,10);z[1]=evallgef(2)+evalvarn(gvar(y));}
     break;
   
   case 17:
@@ -380,10 +380,10 @@ GEN   gaddpex(GEN x, GEN y)
     else e3=pvaluation(x,p,&p2);
     e2=signe((GEN)y[4])?e1+precp(y)-e3:e1-e3;
     z[2]=(long)p;
-    if(e2<=0) {z[3]=un;setprecp(z,0);}
+    if(e2<=0) {z[3]=un;z[1]=evalprecp(0)+evalvalp(e3);}
     else
     {
-      setprecp(z,e2);
+      z[1]=evalprecp(e2)+evalvalp(e3);
       if(e1-e3)
       {
         p1=gpuigs(p,e1-e3);
@@ -391,7 +391,7 @@ GEN   gaddpex(GEN x, GEN y)
       }
       else z[3]=lcopy((GEN)y[3]);
     }
-    z[4]=lmod(p2,(GEN)z[3]);setvalp(z,e3);
+    z[4]=lmod(p2,(GEN)z[3]);
     tetpil=avma;z=gerepile(av,tetpil,gadd(z,y));
   }
   return z;
@@ -873,9 +873,7 @@ GEN     gsubst(GEN x, long v, GEN y)
         if(vx<v) z=gcopy(x);
         else
         {
-          z=cgetg(3,11);
-          z[1]=HIGHVALPBIT+ex*gval(y,v);z[2]=zero;
-          setvarn(z,vx);
+          z=cgetg(3,11);z[1]=evalvalp(ex*gval(y,v))+evalvarn(vx);z[2]=zero;
         }
         return z;
       }
@@ -899,9 +897,7 @@ GEN     gsubst(GEN x, long v, GEN y)
 	ey=valp(y);vy=varn(y);
 	if (ey<1)
 	  {
-	    z=cgetg(3,11);z[2]=zero;
-	    z[1]=HIGHVALPBIT+ey*(ex+lx-2);
-	    setvarn(z,vy);
+	    z=cgetg(3,11);z[1]=evalvalp(ey*(ex+lx-2))+evalvarn(vy);z[2]=zero;
 	  }
 	else
 	  {
@@ -916,7 +912,7 @@ GEN     gsubst(GEN x, long v, GEN y)
 	      }
 	    if(vy!=vx)
 	      {
-		av=avma;z=cgetg(2,11);z[1]=HIGHVALPBIT;setvarn(z,vy);
+		av=avma;z=cgetg(2,11);z[1]=HIGHVALPBIT+evalvarn(vy);
 		for(i=lx-1;i>=2;i--)
 		  {p1=gmul(y,z);av2=avma;z=gadd((GEN)x[i],p1);}
 		if (ex)
@@ -965,7 +961,7 @@ GEN     gsubst(GEN x, long v, GEN y)
 		    else av3=av;
 		    av6=avma;
 		  }
-		z[1]=evalsigne(1)+HIGHVALPBIT;setvarn(z,varn(y));
+		z[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(varn(y));
 		if (ex)
 		  {
 		    if (l<ly) setlg(y,l);
@@ -984,9 +980,8 @@ GEN     gsubst(GEN x, long v, GEN y)
         case 14:
           if(gcmp0(y))
           {
-            z=cgetg(lx,tx);z[1]=evalsigne(1)+HIGHVALPBIT;
-            z[2]=lcopy((GEN)x[2]);setvarn(z,v);
-            for(i=3;i<lx;i++) z[i]=zero;
+            z=cgetg(lx,tx);z[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(v);
+            z[2]=lcopy((GEN)x[2]);for(i=3;i<lx;i++) z[i]=zero;
           }
           else
           {
@@ -1069,8 +1064,7 @@ GEN     recip(GEN x)
       y[i+1]=ldivgs((GEN)u[i+1],i);av7=avma;
     }
     y[lx-1]=lpile(av5,av6,(GEN)y[lx-1]);
-    y[1]=evalsigne(1)+HIGHVALPBIT+1;y=gerepile(av1,av2,y);
-    setvarn(y,v);
+    y[1]=evalsigne(1)+evalvalp(1)+evalvarn(v);y=gerepile(av1,av2,y);
   }
   else
   {
@@ -1109,14 +1103,13 @@ GEN     deriv(GEN x, long v)
         l=avma;
         for(i=lx;(i>=2)&&(isexactzero(p1=deriv((GEN)x[i],v)));avma=l,i--);
         y=cgetg(i+1,10);
-        if(i==1) y[1]=2;
+        if(i==1) y[1]=evallgef(2)+evalvarn(vx);
         else
         {
-          y[1]=evalsigne(1)+evallgef(i+1);y[i]=(long)p1;f=gcmp0(p1);
+	  y[i]=(long)p1;f=gcmp0(p1);
           for(j=2;j<i;j++) {y[j]=lderiv((GEN)x[j],v);if(f) f=gcmp0((GEN)y[j]);}
-	  if(f) setsigne(y,0);
+	  y[1]=f?evallgef(i+1)+evalvarn(vx):evalsigne(1)+evallgef(i+1)+evalvarn(vx);
         }
-        setvarn(y,vx);
         return y;
       }
       if(lx<3) y=gzero;
@@ -1125,14 +1118,14 @@ GEN     deriv(GEN x, long v)
         l=avma;
         for(i=lx-1;(i>=2)&&isexactzero(p1=gmulsg(i-1,(GEN)x[i+1]));avma=l,i--);
         y=cgetg(i+1,tx);
-        if(i==1) y[1]=2;
+        if(i==1) y[1]=evallgef(2)+evalvarn(v);
         else
         {
-          y[1]=evalsigne(1)+evallgef(i+1);y[i]=(long)p1;f=gcmp0(p1);
-          for(i--;i>=2;i--) {y[i]=lmulsg(i-1,(GEN)x[i+1]);if(f) f=gcmp0((GEN)y[i]);}
-	  if(f) setsigne(y,0);
+	  y[i]=(long)p1;f=gcmp0(p1);
+          for(j=i-1;j>=2;j--) 
+	    {y[j]=lmulsg(j-1,(GEN)x[j+1]);if(f) f=gcmp0((GEN)y[j]);}
+          y[1]=f?evallgef(i+1)+evalvarn(v):evalsigne(1)+evallgef(i+1)+evalvarn(v);
         }
-        setvarn(y,v);
       }
       break;
     case 11: vx=varn(x);
@@ -1148,8 +1141,8 @@ GEN     deriv(GEN x, long v)
           if(i==lx) y=ggrando(polx[vx],e+lx-2);
           else
           {
-            y=cgetg(lx-i+2,11);y[1]=HIGHVALPBIT-2+e+i;
-            setvarn(y,vx);y[2]=(long)p1;
+            y=cgetg(lx-i+2,11);y[1]=evalvalp(e+i-2)+evalvarn(vx);
+            y[2]=(long)p1;
             for(j=3;j<=lx-i+1;j++)
               y[j]=lderiv((GEN)x[i+j-2],v);
           }
@@ -1158,32 +1151,26 @@ GEN     deriv(GEN x, long v)
       }
       ly=lx-1;if(ly<3) ly=3;
       if(gcmp0(x))
-        {y=cgetg(3,tx);y[1]=HIGHVALPBIT-1+e;}
+        {y=cgetg(3,tx);y[1]=evalvalp(e-1)+evalvarn(vx);}
       else
       {
         if(e)
         {
-          y=cgetg(lx,tx);
-          for(i=2;i<lx;i++)
-            y[i]=lmulsg(i+e-2,(GEN)x[i]);
-          y[1]=evalsigne(1)+HIGHVALPBIT+e-1;
+          y=cgetg(lx,tx);y[1]=evalsigne(1)+evalvalp(e-1)+evalvarn(vx);
+          for(i=2;i<lx;i++) y[i]=lmulsg(i+e-2,(GEN)x[i]);
         }
         else
         {
-          i=3;
-          while((i<lx)&&gcmp0((GEN)x[i])) i++;
-          if(i==lx)
-            {y=cgetg(ly,tx);y[1]=HIGHVALPBIT-3+lx;}
+          i=3;while((i<lx)&&gcmp0((GEN)x[i])) i++;
+          if(i==lx) {y=cgetg(ly,tx);y[1]=evalvalp(lx-3)+evalvarn(vx);}
           else
           {
             ly=ly-i+3;y=cgetg(ly,tx);
-            y[1]=evalsigne(1)+HIGHVALPBIT-3+i;
-            for(j=2;j<ly;j++)
-              y[j]=lmulsg(j+i-4,(GEN)x[i+j-2]);
+            y[1]=evalsigne(1)+evalvalp(i-3)+evalvarn(vx);
+            for(j=2;j<ly;j++) y[j]=lmulsg(j+i-4,(GEN)x[i+j-2]);
           }
         }
       }
-      setvarn(y,vx);
       break;
     case 13:
     case 14: l=avma;y=cgetg(3,tx);y[2]=lmul((GEN)x[2],(GEN)x[2]);
@@ -1222,7 +1209,7 @@ GEN     integ(GEN x, long v)
     if(gcmp0(x)) y=gzero;
     else
     {
-      y=cgetg(4,10);y[1]=evalsigne(1)+evallgef(4);setvarn(y,v);
+      y=cgetg(4,10);y[1]=evalsigne(1)+evallgef(4)+evalvarn(v);
       y[2]=zero;y[3]=lcopy(x);
     }
     return y;
@@ -1234,13 +1221,12 @@ GEN     integ(GEN x, long v)
     case 10: vx=varn(x);lx=lgef(x);
       if(lx==2)
       {
-        y=cgetg(3,10);y[1]=2;
-        if(vx>v) setvarn(y,v);else setvarn(y,vx);
-        return y;
+        y=cgetg(3,10);y[1]=evallgef(2)+evalvarn(min(v,vx));return y;
       }
       if(vx>v)
       {
-        y=cgetg(4,10);y[1]=(signe(x))?evalsigne(1)+evallgef(4):evallgef(4);setvarn(y,v);
+        y=cgetg(4,10);
+	y[1]=(signe(x))?evalsigne(1)+evallgef(4)+evalvarn(v):evallgef(4)+evalvarn(v);
         y[2]=zero;y[3]=lcopy(x);
         return y;
       }
@@ -1254,20 +1240,20 @@ GEN     integ(GEN x, long v)
       y=cgetg(lx+1,tx);y[2]=zero;
       for(i=3;i<=lx;i++)
         y[i]=ldivgs((GEN)x[i-1],i-2);
-      y[1]=signe(x)?evalsigne(1)+evallgef(lx+1):evallgef(lx+1);setvarn(y,v);
+      y[1]=signe(x)?evalsigne(1)+evallgef(lx+1)+evalvarn(v):evallgef(lx+1)+evalvarn(v);
       break;
     case 11: lx=lg(x);e=valp(x);vx=varn(x);
       if(!signe(x))
       {
         y=cgetg(3,tx);
-        if(vx==v) y[1]=HIGHVALPBIT+1+e;
+        if(vx==v) y[1]=evalvalp(e+1);
         else y[1]=x[1];
         if(vx>v) setvarn(y,v);
         return y;
       }
       if(vx>v)
       {
-        y=cgetg(4,10);y[1]=evalsigne(1)+evallgef(4);setvarn(y,v);
+        y=cgetg(4,10);y[1]=evalsigne(1)+evallgef(4)+evalvarn(v);
         y[2]=zero;y[3]=lcopy(x);
         return y;
       }
@@ -1557,11 +1543,11 @@ GEN gtopoly(GEN x, long v)
   long tx=typ(x),lx,i,j;
   GEN y;
   
-  if(gcmp0(x)) {y=cgetg(2,10);y[1]=2;setvarn(y,v);return y;}
+  if(gcmp0(x)) {y=cgetg(2,10);y[1]=evallgef(2)+evalvarn(v);return y;}
   if(tx<10)
     {
-      y=cgetg(3,10);y[1]=evalsigne(1)+evallgef(3);y[2]=lcopy(x);
-      setvarn(y,v);return y;
+      y=cgetg(3,10);y[1]=evalsigne(1)+evallgef(3)+evalvarn(v);y[2]=lcopy(x);
+      return y;
     }
   switch(tx)
     {
@@ -1585,11 +1571,11 @@ GEN gtopolyrev(GEN x, long v)
   long tx=typ(x),lx,i,j;
   GEN y;
   
-  if(gcmp0(x)) {y=cgetg(2,10);y[1]=evallgef(2);setvarn(y,v);return y;}
+  if(gcmp0(x)) {y=cgetg(2,10);y[1]=evallgef(2)+evalvarn(v);return y;}
   if(tx<10)
     {
-      y=cgetg(3,10);y[1]=evalsigne(1)+evallgef(3);y[2]=lcopy(x);
-      setvarn(y,v);return y;
+      y=cgetg(3,10);y[1]=evalsigne(1)+evallgef(3)+evalvarn(v);y[2]=lcopy(x);
+      return y;
     }
   switch(tx)
     {
@@ -1614,12 +1600,12 @@ GEN gtoser(GEN x, long v)
   GEN y,p1,p2;
   
   if(tx==11) {y=gcopy(x);setvarn(y,v);return y;}
-  if(gcmp0(x)) {y=cgetg(2,11);y[1]=HIGHVALPBIT+precdl;setvarn(y,v);return y;}
+  if(gcmp0(x)) {y=cgetg(2,11);y[1]=evalvalp(precdl)+evalvarn(v);return y;}
   if(tx<10)
     {
-      y=cgetg(precdl+2,11);y[1]=evalsigne(1)+HIGHVALPBIT;y[2]=lcopy(x);
-      for(i=3;i<=precdl+1;i++) y[i]=zero;
-      setvarn(y,v);return y;
+      y=cgetg(precdl+2,11);y[1]=evalsigne(1)+HIGHVALPBIT+evalvarn(v);
+      y[2]=lcopy(x);for(i=3;i<=precdl+1;i++) y[i]=zero;
+      return y;
     }
   switch(tx)
     {
