@@ -114,7 +114,7 @@ extern  entree  fonctions[],**hashtable;
 extern  long    lontyp[],lontyp2[];
 
 extern  jmp_buf environnement;
-extern  FILE    *outfile, *logfile, *infile;
+extern  FILE    *outfile, *logfile, *infile, *errfile;
 
 extern  unsigned long    avma,bot,top;
 extern  GEN     gnil,gun,gdeux,ghalf,gi,gzero;
@@ -129,6 +129,7 @@ extern  long    nvar;         /* numero de la prochaine inconnue */
 extern  long    glbfmt[];
 extern  long    **rectgraph;
 extern  long    pari_randseed;
+extern  long    DEBUGLEVEL;
 
 extern const int STACKSIZE;  /* nombre de gn possibles */
 extern const int TBLSZ;  /* taille de la table de hashcodes */
@@ -148,7 +149,7 @@ extern const double C1;            /* log(2*pi)/2          */
 extern const double C2;             /* 32*log(2)            */
 extern const double C3;     /* log((1+sqrt(5))/2)/(32*log(2)) */
 extern const double C31;           /* 2^31                 */
-extern const int BIGINT;                  /* 2^15-1               */
+extern const long BIGINT;                  /* 2^15-1               */
 extern const long EXP220;            /* 2^20                 */
 extern const long VERYBIGINT;        /* 2^31-1               */
 
@@ -169,13 +170,8 @@ inline double log2(double x) {return log(x)/LOG2;}
 #define labs(x) abs(x)
 #endif
 
-#ifdef __cplusplus
-inline long min(long a, long b) {return a<b ? a : b;}
-inline long max(long a, long b) {return a>b ? a : b;}
-#else
 #define min(a,b) ((a)>(b)?(b):(a))
 #define max(a,b) ((a)>(b)?(a):(b))
-#endif
 
 #define separe(c)     ((c==';')||(c==':')||(c=='\n'))
 
@@ -530,6 +526,8 @@ inline int odd(long x) {return x&1;}
 #define evallgef(x) (x)
 #define evalvarn(x) (((ulong)(x))<<VARNSHIFT)
 #define evalprecp(x) (((long)(x))<<PRECPSHIFT)
+#define evalexpo(x) (HIGHEXPOBIT+(x))
+#define evalvalp(x) (HIGHVALPBIT+(x))
 
 /* alglin.c */
      
@@ -538,7 +536,7 @@ GEN     ker(GEN x),keri(GEN x),kerreel(GEN x, long prec);
 GEN     image(GEN x),imagereel(GEN x, long prec),imagecompl(GEN x),image2(GEN x),suppl(GEN x),eigen(GEN x, long prec),hess(GEN x);
 GEN     carhess(GEN x, long v);
 GEN     gauss(GEN a, GEN b),invmat(GEN a),det(GEN a),detreel(GEN a),det2(GEN a);
-GEN     caract(GEN x, int v),caradj(GEN x, long v, GEN *py),adj(GEN x),caradj0(GEN x, long v),trace(GEN x);
+GEN     caract(GEN x, int v),caradj(GEN x, long v, GEN *py),adj(GEN x),caradj0(GEN x, long v),trace(GEN x),trace9(GEN x,GEN p1);
 GEN     assmat(GEN x),gnorm(GEN x),gnorml2(GEN x),gconj(GEN x),concat(GEN x, GEN y),idmat(long n),conjvec(GEN x,long prec);
 GEN     extract(GEN x, GEN l),matextract(GEN x, GEN l1, GEN l2),gtomat(GEN x),invmulmat(GEN a, GEN b),invmulmatreel(GEN a, GEN b),invmatreel(GEN a);
 GEN     sqred(GEN a),sqred1(GEN a),sqred2(GEN a, long flg),sqred3(GEN a),signat(GEN a),jacobi(GEN a, long prec),matrixqz(GEN x, GEN pp),matrixqz2(GEN x),matrixqz3(GEN x);
@@ -586,13 +584,18 @@ void    lucas(long n, GEN *ln, GEN *ln1);
 
 /* base.c */
 
-GEN     base(GEN x, GEN *y),smallbase(GEN x, GEN *y),discf(GEN x),smalldiscf(GEN x);
-GEN     hnf(GEN x),hnfhavas(GEN x),cleanmod(GEN x,GEN detmat,GEN detmatsur2),hnfmod(GEN x, GEN detmat),smith(GEN x),smith2(GEN x);
+GEN     base(GEN x, GEN *y),smallbase(GEN x, GEN *y),discf(GEN x),smalldiscf(GEN x),discf2(GEN x);
+GEN     hnf(GEN x),hnfhavas(GEN x),cleanmod(GEN x,GEN detmat,GEN detmatsur2);
+GEN     hnfmod(GEN x, GEN detmat),hnfmodid(GEN x,GEN p),smith(GEN x),smith2(GEN x);
 GEN     factoredbase(GEN x, GEN p, GEN *y),factoreddiscf(GEN x, GEN p),allbase(GEN x, long code, GEN *y),galois(GEN x, long prec),initalg(GEN x, long prec),initalg0(GEN x, long prec);
 GEN     tschirnhaus(GEN x),galoisconj(GEN x, long prec),initalg2(GEN x, long prec);
 GEN     primedec(GEN nf,GEN p),idealmul(GEN nf,GEN ix,GEN iy),idealmulred(GEN nf, GEN ix, GEN iy, long prec);
 GEN     idealmulh(GEN nf, GEN ix, GEN iy),element_mulh(GEN nf, long limi, long limj, GEN x, GEN y);
-GEN     idealmulprime(GEN nf,GEN ix,GEN vp),ideallllred(GEN nf,GEN ix,GEN vdir,long prec),minideal(GEN nf,GEN ix,GEN vdir,long prec);
+GEN     idealmulprime(GEN nf,GEN ix,GEN vp),minideal(GEN nf,GEN ix,GEN vdir,long prec);
+GEN     ideallllredall(GEN nf, GEN ix, GEN vdir, long prec, long precint);
+GEN     ideallllred(GEN nf,GEN ix,GEN vdir,long prec);
+GEN     ideallllredpart1(GEN nf,GEN x,GEN vdir,long prec);
+GEN     ideallllredpart2(GEN nf,GEN arch,GEN z,long prec);
 GEN     element_mul(GEN nf,GEN x,GEN y),element_pow(GEN nf,GEN x,GEN k);
 GEN     rootsof1(GEN x),hnfmod(GEN x,GEN detmat),idealinv(GEN nf, GEN ix);
 GEN     idealpow(GEN nf, GEN ix, GEN n),idealpowred(GEN nf, GEN ix, GEN n, long prec);
@@ -601,10 +604,11 @@ GEN     idealadd(GEN nf, GEN x, GEN y), idealdiv(GEN nf, GEN x, GEN y);
 GEN     idealintersect(GEN nf, GEN x, GEN y), principalideal(GEN nf, GEN a);
 GEN     principalidele(GEN nf, GEN a);
 long    idealval(GEN nf,GEN ix,GEN vp), isideal(GEN nf,GEN x);
+long    elementval2(GEN nf, GEN x, GEN d, GEN vp);
 
 GEN     allbase4(GEN f,long code, GEN *y ),base2(GEN x, GEN *y);
 GEN     bsrch(GEN p, GEN fa, long Ka, GEN eta, long Ma),setup(GEN p,GEN f,GEN theta,GEN nut),eleval(GEN f,GEN h,GEN a),vstar(GEN p,GEN h),factcp(GEN p,GEN f,GEN beta),bestnu(GEN w),gcdpm(GEN f1,GEN f2,GEN pm);
-
+GEN     compositum(GEN pol1, GEN pol2);
 
 /* bibli1.c */
      
@@ -635,13 +639,16 @@ GEN     rectrpoint(long ne, GEN x, GEN y),rectbox(long ne, GEN gx2, GEN gy2),rec
 GEN     rectpoints(long ne, GEN listx, GEN listy),rectlines(long ne, GEN listx, GEN listy),rectstring(long ne, GEN x),rectscale(long ne, GEN x1, GEN x2, GEN y1, GEN y2);
 GEN     postdraw(GEN list),postploth(entree *ep, GEN a, GEN b, char *ch),postploth2(entree *ep, GEN a, GEN b, char *ch),postplothraw(GEN listx, GEN listy);
 
-/* buch.c */
+/* buch1.c et buch2.c */
 
-GEN     buchimag(GEN D, GEN gcbach, GEN gcbach2, GEN gCO),buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, GEN gCO, long prec);
-GEN     buchgen(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec),buchgenfu(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec);
-GEN     buchinit(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec);
-GEN     buchinitfu(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, long prec);
-GEN     isprincipal(GEN bignf, GEN x), isunit(GEN bignf, GEN x);
+GEN     buchimag(GEN D, GEN gcbach, GEN gcbach2);
+GEN     buchreal(GEN D, GEN gsens, GEN gcbach, GEN gcbach2, long prec);
+GEN     buchgen(GEN P, GEN gcbach, GEN gcbach2, long prec);
+GEN     buchgenfu(GEN P, GEN gcbach, GEN gcbach2, long prec);
+GEN     buchinit(GEN P, GEN gcbach, GEN gcbach2, long prec);
+GEN     buchinitfu(GEN P, GEN gcbach, GEN gcbach2, long prec);
+GEN     isprincipal(GEN bignf, GEN x);
+GEN     isunit(GEN bignf, GEN x), signunit(GEN bignf);
 int     compte(long **mat, long row, long longueur, long *firstnonzero);
 int     compte2(long **mat, long row, long longueur, long *firstnonzero);
 
@@ -710,6 +717,7 @@ double  rtodbl(GEN x), gtodouble(GEN x);
 /* init.c */
 
 GEN     newbloc(long n),geni(void);
+GEN     allocatemem(unsigned long newsize);
 long    marklist(void);
 #ifdef __cplusplus
 void    init(long parisize, long maxprime, void (*printvar)(long)=printvargp);
@@ -723,7 +731,7 @@ extern "C" void    err(long numerr, ...);
 #else
 void    err(long numerr, ...);
 #endif
-void    recover(long listloc),changevalue(entree *ep, GEN val),allocatemoremem(void);
+void    recover(long listloc),changevalue(entree *ep, GEN val),allocatemoremem(unsigned long newsize);
 
 /* polarit.c */
      
@@ -775,7 +783,7 @@ void    gpolylogz(long m, GEN x, GEN y);
 /* version.c */
 
 GEN     gerepilc(GEN l, GEN p, GEN q);
-void    printversion(void);
+void    printversion(void), printversionno(void);
 
 #ifdef __cplusplus
 extern "C" {
