@@ -82,9 +82,9 @@ int main(long argc, char **argv)
       else usage(argv[0]);
     }
   
+  init(parisize, primelimit);
   printversion();
   pariputs("\nCopyright 1989,1993 by C. Batut, D. Bernardi, H. Cohen and M. Olivier\n\n");
-  init(parisize, primelimit);
   if(tmpparibuffsize) paribuffsize=tmpparibuffsize;
   buffer = (char *)malloc(paribuffsize);
   
@@ -98,7 +98,7 @@ int main(long argc, char **argv)
   champ=0;format='g';
 
   pariputs("Type ? for help\n\n");
-  globs(parisize,primelimit);
+  globs(top-bot,primelimit);
   
   for(;;)
     {
@@ -336,7 +336,7 @@ void escape(char *tch, long parisize, long primelimit)
     case 'b': sor(g[getint(tch, tglobal)], format, -1, champ);pariputc('\n');
       break;
     case 'c': checkok(tch); commands(-1); break;
-    case 'd': checkok(tch); globs(parisize,primelimit); break;
+    case 'd': checkok(tch); globs(top-bot,primelimit); break;
     case 'e': checkok(tch); pariecho = !pariecho; break;
     case 'g': while(isspace(*tch)) tch++;
       for (d = 0; isdigit(*tch);) d = 10 * d + *tch++ - '0';
