@@ -10,6 +10,7 @@ char *helpmessage[] =
   "akell(e,n)=computes the n-th Fourier coefficient of the L-function of the\nelliptic curve e",
   "algdep(x,n)=algebraic relations up to degree n of x",
   "algdep2(x,n,dec)=algebraic relations up to degree n of x where dec is as in \nlindep2",
+  "allocatemem(s)=allocates a new stack of s bytes, or doubles the stack if\nsize is 0, 1 or 2, with a warning if size=0",
   "anell(e,n)=computes the first n Fourier coefficients of the L-function of\nthe elliptic curve e (n<32768)",
   "apell(e,p)=computes a_p for the elliptic curve e using Shanks-Mestre's method",
   "apell2(e,p)=computes a_p for the elliptic curve e using Jacobi symbols",
@@ -34,12 +35,12 @@ char *helpmessage[] =
   "boundcf(x,lmax)=continued fraction expansion of x with at most lmax terms",
   "boundfact(x,lim)=partial factorization of the integer x (using primes up\nto lim)",
   "box(w,x2,y2)=if the cursor is at position (x1,y1), draw a box with diagonal\n(x1,y1) and (x2,y2) in rectwindow w (cursor does not move)",
-  "buchgen(P,c,c2,CO)=compute the structure of the class group and the\nregulator for the number field defined by the polynomial P, where\nfor example c2=c=0.5 and CO=10 (see manual)",
-  "buchgenfu(P,c,c2,CO)=compute the structure of the class group, the regulator\na primitive root of unity and a system of fundamental units for the number\nfield defined by the polynomial P, where for example c2=c=0.5 and CO=10\n(see manual)",
-  "buchimag(D,c,c2,CO)=compute the structure of the class group of the complex\nquadratic field of discriminant D<0, where for example c2=c=0.5 and CO=20\n(see manual)",
-  "buchinit(P,c,c2,CO)=compute the necessary data for future use in ideal and\nunit group computations",
-  "buchinitfu(P,c,c2,CO)=compute the necessary data for future use in ideal and unit\ngroup computations, including fundamental units",
-  "buchreal(D,n,c,c2,CO)=compute the structure of the class group and the\nregulator of the real quadratic field of discriminant D>0, where n=0 (wide)\nn=1 (narrow) and for example c2=c=0.5 and CO=20 (see manual)",
+  "buchgen(P,c,c2)=compute the structure of the class group and the\nregulator for the number field defined by the polynomial P, where\nfor example c2=c=0.3 (see manual)",
+  "buchgenfu(P,c,c2)=compute the structure of the class group, the regulator\na primitive root of unity and a system of fundamental units for the number\nfield defined by the polynomial P, where for example c2=c=0.3 (see manual)",
+  "buchimag(D,c,c2)=compute the structure of the class group of the complex\nquadratic field of discriminant D<0, where for example c2=c=0.5 (see manual)",
+  "buchinit(P,c,c2)=compute the necessary data for future use in ideal and\nunit group computations",
+  "buchinitfu(P,c,c2)=compute the necessary data for future use in ideal and unit\ngroup computations, including fundamental units",
+  "buchreal(D,n,c,c2)=compute the structure of the class group and the\nregulator of the real quadratic field of discriminant D>0, where n=0 (wide)\nn=1 (narrow) and for example c2=c=0.5 (see manual)",
   "bytesize(x)=number of bytes occupied by the complete tree of the object x",
   "ceil(x)=ceiling of x=smallest integer>=x",
   "centerlift(x)=centered lift of x. Same as lift except for integermods",
@@ -129,6 +130,7 @@ char *helpmessage[] =
   "hermite(x)=(upper triangular) Hermite normal form of x, basis for the lattice\nformed by the columns of x",
   "hermitehavas(x)=(upper triangular) Hermite normal form of x, basis for the\nlattice formed by the columns of x, using Havas's algorithm",
   "hermitemod(x,d)=(upper triangular) Hermite normal form of x, basis for the\nlattice formed by the columns of x, where d is the non-zero determinant of this\nlattice",
+  "hermitemodid(x,d)=(upper triangular) Hermite normal form of x concatenated\nwith d times the identity matrix",
   "hess(x)=Hessenberg form of x",
   "hilb(x,y,p)=Hilbert symbol at p of x,y (integers or fractions)",
   "hilbert(n)=Hilbert matrix of order n (n C-integer)",
@@ -185,7 +187,7 @@ char *helpmessage[] =
   "isqrt(x)=integer square root of x (x integer)",
   "issqfree(x)=true(1) if x is squarefree, false(0) if not",
   "issquare(x)=true(1) if x is a square, false(0) if not",
-  "isunit(bnf,x)=bnf being output by buchinit, gives the vector of exponents of x on the\nfundamental units and the roots of unity if x is a unit, the empty vector otherwise",
+  "isunit(bnf,x)=bnf being output by buchinit, gives the vector of exponents of x on\nthe fundamental units and the roots of unity if x is a unit,\nthe empty vector otherwise",
   "jacobi(x)=eigenvalues and orthogonal matrix of eigenvectors of the real\nsymmetric matrix x",
   "jbesselh(n,x)=J-bessel function of index n+1/2 and argument x, where n is a\nnon-negative integer",
   "jell(x)=elliptic j invariant of x",
@@ -355,6 +357,7 @@ char *helpmessage[] =
   "sigmak(k,x)=sum of the k-th powers of the divisors of x (k C-integer)",
   "sign(x)=sign of x, of type integer, real or fraction",
   "signat(x)=signature of the symmetric matrix x",
+  "signunit(bnf)=matrix of signs of the real embeddings of the system of\nfundamental units found by buchinit",
   "simplefactmod(x,p)=same as factmod except that only the degrees of the\nirreducible factors are given",
   "simplify(x)=simplify the object x as much as possible",
   "sin(x)=sine of x",
