@@ -57,9 +57,10 @@ long    avloc;
 long    prec=5, precdl=16, defaultpadicprecision=16;
 long    tglobal,paribuffsize=30000,pariecho=0;
 jmp_buf environnement;
-FILE    *outfile = stdout;
-FILE    *logfile = NULL;
-FILE    *infile = stdin;
+FILE    *outfile;
+FILE    *errfile;
+FILE    *logfile;
+FILE    *infile;
 long    nvar = 0;
 GEN     gnil,gzero,gun,gdeux,ghalf,polvar,gi,RAVYZARC;
 GEN     gpi=(GEN)0;
@@ -133,9 +134,11 @@ void init(long parisize, long maxprime)
 #else
   printvariable=printvargp;
 #endif
+
+  outfile = stdout;errfile = stderr;logfile = NULL;infile = stdin;
   if (setjmp(environnement))
   {
-    fprintf(stderr, "\n  ***   Error in the PARI system. End of the program.\n");
+    fprintf(errfile, "\n  ***   Error in the PARI system. End of the program.\n");
     exit(1);
   }
   signal(SIGINT,catchinterrupt);
@@ -310,25 +313,25 @@ void err(long numerr, ...)
   GEN noninv;
 
   va_start(poer,numerr);
-  fprintf(stderr, "  ***   %s",errmessage[numerr]);
+  fprintf(errfile, "  ***   %s",errmessage[numerr]);
   switch (numerr)
   {
     case matcher1:
       ch=va_arg(poer, char*);c = *ch++;
-      fprintf(stderr, "'%c'\n  ***   instead of: '%s'", c, ch); break;
+      fprintf(errfile, "'%c'\n  ***   instead of: '%s'", c, ch); break;
     case impl: ch=va_arg(poer, char*);
-      fprintf(stderr, " %s is not yet implemented.",ch); break;
+      fprintf(errfile, " %s is not yet implemented.",ch); break;
     case talker: ch=va_arg(poer, char*);
-      fprintf(stderr, "%s.",ch); break;
+      fprintf(errfile, "%s.",ch); break;
     case varer1:
     case unknowner1:
-    case caracer1: ch=va_arg(poer, char*);fprintf(stderr, "'%s'",ch);break;
+    case caracer1: ch=va_arg(poer, char*);fprintf(errfile, "'%s'",ch);break;
     case invmoder: ch=va_arg(poer, char*);noninv=va_arg(poer, GEN);
-      temp=outfile;outfile=stderr;fprintf(stderr,": ");
+      temp=outfile;outfile=errfile;fprintf(errfile,": ");
       brute(noninv,'g',-1);outfile=temp;break;
-    case errpile: putc('\n', stderr);allocatemoremem();break;
+    case errpile: putc('\n', errfile);allocatemoremem(0);break;
   }
-  putc('\n', stderr);
+  putc('\n', errfile);
   va_end(poer);
   longjmp(environnement, numerr);
 }
@@ -372,16 +375,26 @@ void recover(long listloc)
       killbloc(x);
 }
 
-void allocatemoremem(void)
+void allocatemoremem(unsigned long newsize)
 {
   long av,declg,declg2,tl,parisize,v;
   GEN ll,pp,l1,l2,l3;
   unsigned long topold,avmaold,botold;
 
-  avmaold=avloc;topold=top;botold=bot;parisize=(topold-botold)<<1;
+  avmaold=avloc;topold=top;botold=bot;
+  if(newsize<3) parisize=(topold-botold)<<1;
+  else 
+    {
+      if(newsize<(topold-avmaold)) 
+	err(talker,"required stack memory too small");
+      else parisize=newsize+16-((newsize-1)&15+1);
+    }
   if (!(bot=(long)malloc(parisize))) err(nomer2);
-  fprintf(stderr, "  *** Warning: doubling the stack size; new stack = %d\n",parisize);
-  fprintf(stderr, "  *** Please reissue the same command if you are under GP\n");
+  if(!newsize)
+    {
+      fprintf(errfile, "  *** Warning: doubling the stack size; new stack = %d\n",parisize);
+      fprintf(errfile, "  *** Please reissue the same command if you are under GP\n");
+    }
   top=avma=bot+parisize;
   declg=(long)top-(long)topold;declg2=declg>>TWOPOTBYTES_IN_LONG;
   for(ll=(GEN)top,pp=(GEN)topold;pp>(GEN)avmaold;) *--ll= *--pp;
@@ -404,3 +417,9 @@ void allocatemoremem(void)
   free((void *)botold);avloc=avma=av;
 }
 
+GEN allocatemem(unsigned long newsize)
+{
+  allocatemoremem(newsize);
+  longjmp(environnement,errpile);
+  return gnil; /*inutile mais ca fait plaisir a des compilos */
+}
