@@ -17,9 +17,9 @@ GEN   mulsi(long x, GEN y)
   if(x<0) {s= -s;x= -x;}
   z=cgeti(ly+1);hiremainder=0;
   for(i=ly-1;i>=2;i--) z[i+1]=addmul(x,y[i]);
-  if(hiremainder) {z[2]=hiremainder;setlgef(z,ly+1);}
-  else {avma+=BYTES_IN_LONG;z[1]=z[0]-1;z++;setlgef(z,ly);}
-  setsigne(z,s);return z;
+  if(hiremainder) {z[2]=hiremainder;z[1]=evallgef(ly+1)+evalsigne(s);}
+  else {avma+=BYTES_IN_LONG;z[1]=z[0]-1;z++;z[1]=evallgef(ly)+evalsigne(s);}
+  return z;
 }
 
 GEN stoi(long x)
@@ -177,7 +177,7 @@ void affsr(long s, GEN x)
   else
     {
       d=1;if(s<0) {d= -1;s= -s;}
-      l=bfffo(s);setexpo(x,(BITS_IN_LONG-1)-l);setsigne(x,d);
+      l=bfffo(s);x[1]=evalexpo((BITS_IN_LONG-1)-l)+evalsigne(d);
       x[2]=(s<<l);for(i=3;i<lg(x);i++) x[i]=0;
     }
 }
@@ -193,7 +193,7 @@ void affii(GEN x, GEN y)
 
 void affir(GEN x, GEN y)
 {
-  long lx=lgef(x),ly=lg(y),s,i,l,k;
+  long lx=lgef(x),ly=lg(y),s,s1,i,l,k;
   
   s=signe(x);
   if(!s)
@@ -202,24 +202,23 @@ void affir(GEN x, GEN y)
     }
   else
     {
-      setsigne(y,s);l=(lx-2)<<TWOPOTBITS_IN_LONG;s=bfffo(x[2]);
-      if(s)
+      l=(lx-2)<<TWOPOTBITS_IN_LONG;s1=bfffo(x[2]);
+      y[1]=evalsigne(s)+evalexpo(l-s1-1);
+      if(s1)
 	{
-	  setexpo(y,l-s-1);
 	  if(lx<=ly)
 	    {
 	      for(i=lx;i<ly;i++) y[i]=0;k=0;
-	      for(i=lx-1;i>=2;i--) {y[i]=shiftl(x[i],s)+k;k=hiremainder;}
+	      for(i=lx-1;i>=2;i--) {y[i]=shiftl(x[i],s1)+k;k=hiremainder;}
 	    }
 	  else
 	    {
-	      shiftl(x[ly],s);k=hiremainder;
-	      for(i=ly-1;i>=2;i--) {y[i]=shiftl(x[i],s)+k;k=hiremainder;}
+	      shiftl(x[ly],s1);k=hiremainder;
+	      for(i=ly-1;i>=2;i--) {y[i]=shiftl(x[i],s1)+k;k=hiremainder;}
 	    }
 	}
       else
 	{
-	  setexpo(y,l-1);
 	  if(lx<=ly)
 	    {
 	      for(i=lx;i<ly;i++) y[i]=0;
@@ -343,13 +342,13 @@ GEN mptrunc(GEN x)
   if(typ(x)==1) return icopy(x);
   s=signe(x);if(!s) return gzero;
   e=expo(x);if(e<0) return gzero;
-  d=e>>TWOPOTBITS_IN_LONG;m=e&(BITS_IN_LONG-1);if(d>=lx-2) err(truer2);
-  y=cgeti(d+3);y[1]=y[0];setsigne(y,s);
-  if(m==(BITS_IN_LONG-1)) for(i=2;i<=d+2;i++) y[i]=x[i];
+  d=(e>>TWOPOTBITS_IN_LONG)+3;m=e&(BITS_IN_LONG-1);if(d>lx) err(truer2);
+  y=cgeti(d);y[1]=evallgef(d)+evalsigne(s);
+  if(m==(BITS_IN_LONG-1)) for(i=2;i<d;i++) y[i]=x[i];
   else
     {
       m++;p1=0;
-      for(i=2;i<=d+2;i++)
+      for(i=2;i<d;i++)
 	{
 	  p2=shiftl(x[i],m);y[i]=hiremainder+p1;p1=p2;
 	}
@@ -367,18 +366,18 @@ GEN mpent(GEN x)
   if(signe(x)>=0) return mptrunc(x);
   e=expo(x);
   if(e<0) {y=cgeti(3);y[2]=1;y[1]=evalsigne(-1)+evallgef(3);return y;}
-  d=e>>TWOPOTBITS_IN_LONG;m=e&(BITS_IN_LONG-1);if(d>=lx-2) err(truer2);
-  y=cgeti(d+3);y[1]=evalsigne(-1)+evallgef(d+3);
+  d=(e>>TWOPOTBITS_IN_LONG)+3;m=e&(BITS_IN_LONG-1);if(d>lx) err(truer2);
+  y=cgeti(d);y[1]=evalsigne(-1)+evallgef(d);
   if(m==(BITS_IN_LONG-1)) 
     {
-      for(i=2;i<=d+2;i++) y[i]=x[i];
+      for(i=2;i<d;i++) y[i]=x[i];
       while((i<lx)&&(!x[i])) i++;
       f=(i<lx);
     }    
   else
     {
       m++;p1=0;
-      for(i=2;i<=d+2;i++)
+      for(i=2;i<d;i++)
 	{
 	  p2=shiftl(x[i],m);y[i]=hiremainder+p1;p1=p2;
 	}
@@ -391,7 +390,7 @@ GEN mpent(GEN x)
     }
   if(f)
     {
-      for(i=d+2;(i>=2)&&(y[i]==(long)MAXULONG);i--) y[i]=0;
+      for(i=d-1;(i>=2)&&(y[i]==(long)MAXULONG);i--) y[i]=0;
       if(i>=2) y[i]++;
       else
 	{
@@ -620,7 +619,8 @@ GEN addii(GEN x, GEN y)
       else
 	{
 	  for(j=3;(j<lx)&&(!z[j]);j++);
-	  i=j-2;z[i+1]=z[i]=z[0]-i;z+=i;avma+=(i<<TWOPOTBYTES_IN_LONG);setsigne(z,sx);
+	  i=j-2;z[i+1]=z[i]=z[0]-i;z+=i;avma+=(i<<TWOPOTBYTES_IN_LONG);
+	  setsigne(z,sx);
 	}
     }
   return z;
@@ -690,7 +690,7 @@ GEN addrr(GEN x, GEN y)
   if(e)
     {
       if(e<0) {z=x;x=y;y=z;lz=lx;lx=ly;ly=lz;ey=ex;e= -e;sz=sx;sx=sy;sy=sz;}
-      d=e>>TWOPOTBITS_IN_LONG;m=e&(BITS_IN_LONG-1);
+      d=(e>>TWOPOTBITS_IN_LONG);m=e&(BITS_IN_LONG-1);
       if(d>=ly-2) return rcopy(y);
       l=d+lx;
       if(l>=ly)
@@ -903,9 +903,12 @@ GEN mulss(long x, long y)
   if((!x)||(!y)) return gzero;
   s=1;if(x<0) {s= -1;x= -x;} if(y<0) {s= -s;y= -y;}
   p1=mulll(x,y);
-  if(hiremainder) {z=cgeti(4);z[2]=hiremainder;z[3]=p1;}
-  else {z=cgeti(3);z[2]=p1;}
-  z[1]=z[0];setsigne(z,s);return z;
+  if(hiremainder) 
+    {
+      z=cgeti(4);z[1]=evalsigne(s)+evallgef(4);z[2]=hiremainder;z[3]=p1;
+    }
+  else {z=cgeti(3);z[1]=evalsigne(s)+evallgef(3);z[2]=p1;}
+  return z;
 }
 
 GEN mulsr(long x, GEN y)
@@ -923,7 +926,7 @@ GEN mulsr(long x, GEN y)
   else
     {
       if(x==1) {z=rcopy(y);setsigne(z,s);return z;}
-      lx=lg(y);z=cgetr(lx);setsigne(z,s);
+      lx=lg(y);z=cgetr(lx);
       p2=mulll(x,y[lx-1]);
       for(i=lx-2;i>=2;i--) z[i+1]=addmul(x,y[i]);
       z[2]=hiremainder;p1=bfffo(hiremainder);
@@ -936,7 +939,7 @@ GEN mulsr(long x, GEN y)
 	    }
 	}
       e=BITS_IN_LONG-p1+expo(y);if(e>=HIGHEXPOBIT) err(muler2);
-      setexpo(z,e);
+      z[1]=evalsigne(s)+evalexpo(e);
     }  
   return z;
 }
@@ -951,7 +954,7 @@ GEN mulii(GEN x, GEN y)
   if(sy<0) sx= -sx;
   if(lx>ly) {z=x;x=y;y=z;lz=lx;lx=ly;ly=lz;}
   lz=lx+ly-2;if(lz>LGBITS) err(muler1);
-  z=cgeti(lz);z[1]=z[0];setsigne(z,sx);
+  z=cgeti(lz);z[1]=evallgef(lz)+evalsigne(sx);
   p1=x[lx-1];hiremainder=0;
   for(i=ly-1;i>=2;i--) z[lx+i-2]=addmul(p1,y[i]);
   z[lx-1]=hiremainder;
@@ -1046,7 +1049,7 @@ GEN mulir(GEN x, GEN y)
       z[1]=e;return z;
     }
   lz=lg(y);if(sy<0) sx= -sx;
-  z=cgetr(lz);setsigne(z,sx);av=avma;
+  z=cgetr(lz);av=avma;
   temp=cgetr(lz+1);affir(x,temp);x=y;y=temp;
   e=expo(y)+ey+HIGHEXPOBIT;if(e>=EXPOBITS) err(muler4);
   if(e<0) err(muler5);
@@ -1344,11 +1347,15 @@ GEN divis(GEN y, long x)
   if((ulong)x>(ulong)y[2])
     {
       if(ly==3) {hiremainder=itos(y);return gzero;}
-      else {z=cgeti(ly-1);d=1;hiremainder=y[2];}
+      else 
+	{
+	  z=cgeti(ly-1);z[1]=evallgef(ly-1)+evalsigne(s);
+	  d=1;hiremainder=y[2];
+	}
     }
-  else {z=cgeti(ly);d=0;hiremainder=0;}
+  else {z=cgeti(ly);z[1]=evallgef(ly)+evalsigne(s);d=0;hiremainder=0;}
   for(i=d+2;i<ly;i++) z[i-d]=divll(y[i],x);
-  z[1]=z[0];setsigne(z,s);if(s<0) hiremainder= -((long)hiremainder);
+  if(s<0) hiremainder= -((long)hiremainder);
   return z;
 }
 
@@ -1531,10 +1538,11 @@ GEN divrs(GEN x, long y)
     }
   if(y<0) {s= -s;y= -y;}
   if(y==1) {z=rcopy(x);setsigne(z,s);return z;}
-  z=cgetr(lx=lg(x));setsigne(z,s);hiremainder=0;
+  z=cgetr(lx=lg(x));hiremainder=0;
   for(i=2;i<lx;i++) z[i]=divll(x[i],y);
-  garde=divll(0,y);sh=bfffo(z[2]);ex=expo(x)-sh;if((-ex)>HIGHEXPOBIT) err(diver7);
-  setexpo(z,ex);shiftl(garde,sh);k=hiremainder;
+  garde=divll(0,y);sh=bfffo(z[2]);ex=expo(x)-sh;
+  if((-ex)>HIGHEXPOBIT) err(diver7);
+  z[1]=evalsigne(s)+evalexpo(ex);shiftl(garde,sh);k=hiremainder;
   for(i=lx-1;i>=2;i--) {z[i]=shiftl(z[i],sh)+k;k=hiremainder;}
   return z;
 }
@@ -1561,7 +1569,7 @@ int divise(GEN x, GEN y)
 
 GEN dvmdii(GEN x, GEN y, GEN *z)
 {
-  long av,av2,lx,ly,lz,i,j,dec,sh,k,k1,sx=signe(x),sy=signe(y);
+  long av,av2,lx,ly,lz,lp3,i,j,dec,sh,k,k1,sx=signe(x),sy=signe(y);
   long saux,k3,k4,av1,flk4;
   ulong si,qp;
   GEN p1,p2,p3,p4;
@@ -1632,7 +1640,11 @@ GEN dvmdii(GEN x, GEN y, GEN *z)
 	    {
 	      k1=mulll(qp,saux);k3=subll(k1,p1[i+2]);k+=overflow;
 	      flk4=((ulong)hiremainder>(ulong)k);k4=subll(hiremainder,k);
-	      while(flk4) {qp--;k3=subll(k3,saux);k4-=overflow;flk4=((ulong)k4>(ulong)si);k4=subll(k4,si);}
+	      while(flk4) 
+		{
+		  qp--;k3=subll(k3,saux);k4-=overflow;
+		  flk4=((ulong)k4>(ulong)si);k4=subll(k4,si);
+		}
 	    }
 	  hiremainder=0;
 	  for(j=ly-1;j>=2;j--)
@@ -1652,14 +1664,14 @@ GEN dvmdii(GEN x, GEN y, GEN *z)
 	{
 	  if(p1[1])
 	    {
-	      p3=cgeti(lz+3);for(j=2;j<=lz+2;j++) p3[j]=p1[j-1];
+	      p3=cgeti(lp3=lz+3);for(j=2;j<lp3;j++) p3[j]=p1[j-1];
 	    }
 	  else
 	    {
-	      p3=cgeti(lz+2);
-	      if(!lz) sy=0;else for(j=2;j<=lz+1;j++) p3[j]=p1[j];
+	      p3=cgeti(lp3=lz+2);
+	      if(!lz) sy=0;else for(j=2;j<lp3;j++) p3[j]=p1[j];
 	    }
-	  if(lg(p3)<3) p3[1]=2;else {p3[1]=p3[0];setsigne(p3,sy);}
+	  if(lp3<3) p3[1]=evallgef(2);else {p3[1]=evallgef(lp3)+evalsigne(sy);}
 	}
       if(z!=0)
 	{
@@ -1667,12 +1679,17 @@ GEN dvmdii(GEN x, GEN y, GEN *z)
 	  if(j==lx) p4=gzero;
 	  else
 	    {
-	      p4=cgeti(lx-j+2);p4[1]=p4[0];
+	      p4=cgeti(lp3=lx-j+2);p4[1]=p4[0];
 	      if(!sh) for(i=2;j<lx;j++,i++) p4[i]=p1[j];
 	      else
 		{
 		  hiremainder=0;k1=shiftlr(p1[j++],sh);k=hiremainder;
-		  if(k1) {p4[2]=k1;dec=1;} else {p4[1]=p4[0]-1;p4++;avma+=BYTES_IN_LONG;p4[1]=p4[0];dec=0;}
+		  if(k1) {p4[2]=k1;dec=1;} 
+		  else 
+		    {
+		      p4[1]=p4[0]-1;p4++;avma+=BYTES_IN_LONG;
+		      p4[1]=p4[0];dec=0;
+		    }
 		  for(i=2+dec;j<lx;j++,i++)
 		    {
 		      p4[i]=shiftlr(p1[j],sh)+k;k=hiremainder;
@@ -1683,7 +1700,8 @@ GEN dvmdii(GEN x, GEN y, GEN *z)
 	}
       if((long)z==(long)MAXULONG) return gerepile(av,av1,p4);
       if((long)z==0) return gerepile(av,av1,p3);
-      av2=avma;dec=lpile(av,av1,0)>>TWOPOTBYTES_IN_LONG;*z=adecaler(p4,av1,av2)?p4+dec:p4;
+      av2=avma;dec=lpile(av,av1,0)>>TWOPOTBYTES_IN_LONG;
+      *z=adecaler(p4,av1,av2)?p4+dec:p4;
       return adecaler(p3,av1,av2)?p3+dec:p3;
     }
 }
@@ -1798,14 +1816,16 @@ GEN dbltor(double x)
   int n;
   
   if(x==0) {z=cgetr(3);z[2]=0;z[1]=HIGHEXPOBIT-308;return z;}
-  z=cgetr(4);if(x<0) {setsigne(z,-1);x= -x;} else setsigne(z,1);
-  ex=floor(log2(x));setexpo(z,ex);x=x*exp2(-ex+co-1);y=x;x-=y;z[2]=y;
+  z=cgetr(4);ex=floor(log2(x));
+  if(x<0) {z[1]=evalexpo(ex)+evalsigne(-1);x= -x;}
+  else z[1]=evalexpo(ex)+evalsigne(1);
+  x=x*exp2(-ex+co-1);y=x;x-=y;z[2]=y;
   y=exp2(co)*x;z[3]=y;
 
   if(!(z[2]&HIGHBIT))
     {
       n=bfffo(z[2]);z[2]=shiftl(z[2],n);z[3]=shiftl(z[3],n);
-      z[2]+=hiremainder;setexpo(z,expo(z)-n);
+      z[2]+=hiremainder;setexpo(z,ex-n);
     }
   return z;
 }
