@@ -1525,7 +1525,7 @@ void splitgen(GEN m, GEN *t, long d, GEN  p, GEN q)
 ------------------------------------------------------*/
 
 {  
-  long j,l,v,av,av1,dv;
+  long j,l,v,av,av1,dv,dec;
   GEN w,wm,unmodp;
 
   if ((dv=lgef(*t)-3)==d) return;
@@ -1539,9 +1539,9 @@ void splitgen(GEN m, GEN *t, long d, GEN  p, GEN q)
       wm=gpui(gmodulcp(w,*t),q);
       w=gsub((GEN)wm[2],unmodp);
       av1=avma;
-      m=gcopy(m);
-      w=gerepile(av,av1,ggcd(*t,w));
-      l=lgef(w)-3;
+      m=gcopy(m);w=ggcd(*t,w);
+      dec=lpile(av,av1,0)>>TWOPOTBYTES_IN_LONG;
+      m+=dec;w+=dec;l=lgef(w)-3;
     }
   while((!l)||(l==dv));
   t[j=l/d]=gdiv(*t,w);*t=w;
@@ -1888,7 +1888,7 @@ GEN padicff2(GEN nf,GEN p,long pr)
       vecteur=element_pow(nf,(GEN)((GEN)(dec_p[i]))[2],(GEN)((GEN)(dec_p[i]))[3]);
       mat_pe=cgetg(N+1,19);
       for(j=1;j<=N;j++) mat_pe[j]=(long)element_mul(nf,(GEN)ident[j],vecteur);
-      mat_pe=hnf(concat(gmul(p,ident),mat_pe));
+      mat_pe=hnfmodid(concat(mat_pe,gmul(p,ident)),p);
       mat_pke=mat_pe;
       for(j=1;j<=pr-1;j++) mat_pke=idealmul(nf,mat_pe,mat_pke);
       dim_pke=lg(mat_pke)-1;
