@@ -1026,7 +1026,7 @@ GEN resultant2(GEN x, GEN y)
 
 GEN  srgcd(GEN x, GEN y)
 {
-  long  degq,av,av1,tetpil,f,vx,tx=typ(x),ty=typ(y),dx,dy,du,dv,dr,com;
+  long  degq,av,av1,tetpil,f,vx,tx=typ(x),ty=typ(y),dx,dy,du,dv,dr;
   GEN d,g,h,r,p1,p2,p3,p4,u,v;
 
   if(gcmp0(x)||gcmp0(y)) return gzero;
@@ -1034,39 +1034,29 @@ GEN  srgcd(GEN x, GEN y)
   if((tx!=10)||(ty!=10)) err(subrer1);
   if((vx=varn(x))!=varn(y)) return gun;
   dx=lgef(x);dy=lgef(y);
-  if (dx<dy)
-  {p1=x;x=y;y=p1;f=dx;dx=dy;dy=f;}
-  av=avma;
-  p3=content(x);p4=content(y);d=ggcd(p3,p4);
+  if (dx<dy) {p1=x;x=y;y=p1;f=dx;dx=dy;dy=f;}
+  av=avma;p3=content(x);p4=content(y);d=ggcd(p3,p4);
   tetpil=avma;d=gmul(d,polun[vx]);
   if(dy==3) return gerepile(av,tetpil,d);
-  av1=avma;u=gdiv(x,p3);v=gdiv(y,p4);g=gun;h=gun;f=1;com=0;
+  av1=avma;u=gdiv(x,p3);v=gdiv(y,p4);g=gun;h=gun;f=1;
   while (f==1)
   {
-    du=lgef(u);dv=lgef(v);degq=du-dv;com++;
+    du=lgef(u);dv=lgef(v);degq=du-dv;
     r=psres(u,v);dr=lgef(r);
     if(dr<=3) {f=(dr==3)?2:0;}
     else
     {
-      if(com==10)
-      {
-        com=0;u=gdiv(v,content(v));v=gdiv(r,content(r));
-        g=gun;h=gun;
-      }
-      else
-      {
-        u=v;
-        if(degq==1) p1=gmul(h,g);
-        else p1=gmul(gpuigs(h,degq),g);
-        v=gdiv(r,p1);
-        g=(GEN)u[lgef(u)-1];
-        if(degq==1) h=g;
-        else if(degq)
+      u=v;
+      if(degq==1) p1=gmul(h,g);
+      else p1=gmul(gpuigs(h,degq),g);
+      v=gdiv(r,p1);
+      g=(GEN)u[lgef(u)-1];
+      if(degq==1) h=g;
+      else if(degq)
         {
           p1=gpuigs(g,degq);p2=gpuigs(h,degq-1);
           h=gdiv(p1,p2);
         }
-      }
     }
   }
   if(f) {avma=av1;return gerepile(av,tetpil,d);}
@@ -1074,7 +1064,6 @@ GEN  srgcd(GEN x, GEN y)
   return gerepile(av,tetpil,gmul(d,p1));
 }
 
-
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 /*                                                                 */
@@ -1083,7 +1072,6 @@ GEN  srgcd(GEN x, GEN y)
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
-
 GEN   psres(GEN x, GEN y)
 {
   long  l,tetpil,degx,degy;
@@ -1099,30 +1087,30 @@ GEN   psres(GEN x, GEN y)
   return z;
 }
 
-
 GEN   discsr(GEN x)
 {
   long  dx,av=avma,tetpil,tx=typ(x),i;
   GEN z,p1,p2;
 
   switch(tx)
-  {
-  case 6: z=stoi(-4);break;
-  case 8: z=discsr((GEN)x[1]);break;
-  case 10: dx=lgef(x);p1=deriv(x,varn(x));
-    p1=subres(x,p1);tetpil=avma;p1=gdiv(p1,(GEN)x[dx-1]);
-    if (((dx-3)&3)>1)
-    {tetpil=avma;z=gerepile(av,tetpil,gneg(p1));}
-    else z=gerepile(av,tetpil,p1);break;
-  case 9: z=discsr((GEN)x[1]);break;
-  case 15:
-  case 16: p1=mulii((GEN)x[2],(GEN)x[2]);p2=shifti(mulii((GEN)x[1],(GEN)x[3]),2);tetpil=avma;
-    z=gerepile(av,tetpil,subii(p1,p2));break;
-  case 17: case 18: case 19:
-    dx=lg(x);z=cgetg(dx,tx);for(i=1;i<dx;i++) z[i]=(long)discsr((GEN)x[i]);
-    break;
-  default: err(discsrer1);
-  }
+    {
+    case 6: z=stoi(-4);break;
+    case 8: z=discsr((GEN)x[1]);break;
+    case 10: dx=lgef(x);p1=deriv(x,varn(x));
+      p1=subres(x,p1);tetpil=avma;p1=gdiv(p1,(GEN)x[dx-1]);
+      if (((dx-3)&3)>1)
+	{tetpil=avma;z=gerepile(av,tetpil,gneg(p1));}
+      else z=gerepile(av,tetpil,p1);break;
+    case 9: z=discsr((GEN)x[1]);break;
+    case 15:
+    case 16: p1=mulii((GEN)x[2],(GEN)x[2]);
+      p2=shifti(mulii((GEN)x[1],(GEN)x[3]),2);tetpil=avma;
+      z=gerepile(av,tetpil,subii(p1,p2));break;
+    case 17: case 18: case 19:
+      dx=lg(x);z=cgetg(dx,tx);for(i=1;i<dx;i++) z[i]=(long)discsr((GEN)x[i]);
+      break;
+    default: err(discsrer1);
+    }
   return z;
 }
 
