@@ -755,9 +755,9 @@ GEN palogaux(GEN x)
     y=gaddgs(x,-1);
     if(!cmpis((GEN)x[2],2))
     {
-      setvalp(y,valp(y)-1);mpshiftz((GEN)y[3],-1,(GEN)y[3]);
+      setvalp(y,valp(y)-1);y[3]=(long)shifti((GEN)y[3],-1);
     }
-    return y;
+    tetpil=avma;return gerepile(av1,tetpil,gcopy(y));
   }
   y=gdiv(gaddgs(x,-1),gaddgs(x,1));e=valp(y);pp=precp(y);
   if(cmpis((GEN)x[2],2))
