@@ -979,9 +979,9 @@ GEN sagm(GEN x, long prec)
 	  a=a1;b=b1;
 	  a1=gmul2n(gadd(a,b),-1);b1=gsqrt(gmul(a,b),0);
 	  p1=gsub(b1,a1);ep=valp(p1)-valp(b1);
-	  if(ep<=0) {b1=gneg(b1);p1=gsub(b1,a1);ep=valp(p1)-valp(b1);}
+/*	  if(ep<=0) {b1=gneg(b1);p1=gsub(b1,a1);ep=valp(p1)-valp(b1);} */
 	}
-      while(ep<pp);
+      while(ep<pp&&(!gcmp0(p1)));
       tetpil=avma;z=gerepile(av,tetpil,gcopy(a1));break;
     case 3: err(impl,"agm of mod");
     default: z=transc(sagm,x,prec);
